importPath=string.gsub(getfenv(1)._.Name,"%.Main","").."."
resourcePath=string.gsub(importPath,"%.","/").."Resources/"
-- to do
-- should add events for capacity changes but the next refresh will automatically adjust anyway

-- ItemCategory needs to be updated with the new RoI and RoR categories
-- need to continue testing the Undefined category. it seemed that all items got put into the undefined category once but I have not been able to replicate it.

-- this file contains the initialization, inventory maintenance and event handling, settings and data save/load functionality
-- I may at some point remerge the itemtracker functionality since 75% or more of the functionality is actually replicated here
-- the display functionality is all handled in the inventoryWindow.lua file

-- the quantities are stored in the accountItemQty structure:
-- -- accountItemQty[itemName].Total = total for this item
-- -- accountItemQty[itemName].Category = the category code for this item, used for filtering
-- -- accountItemQty[itemName].SortCategory = the category code for this item, used for sorting in case the Category code is not yet defined
-- -- accountItemQty[itemName].Quality = the quality code for this item, used for sorting
-- -- accountItemQty[itemName].BackgroundImageID = image resource ID for the icon background. used to display an icon for items not currently accessible by the current character
-- -- accountItemQty[itemName].IconImageID = image resource ID for the icon foreground. used to display an icon for items not currently accessible by the current character
-- -- accountItemQty[itemName].ListItem[storage] = handle to an entry in inventoryPanel.ItemList representing this item
-- -- accountItemQty[itemName].ListItem[charName][slot] = handle to an entry in inventoryPanel.ItemList representing this item
-- -- accountItemQty[name].Qty[storage].Subtotal = quantity in the specified storage
-- -- accountItemQty[name].Qty[storage][chest/bag] = detail quantity in the specified chest/bag of the specified storage
-- -- accountItemQty[name].Qty[charName][slot] = quantity in the specified slot of the current player bags


-- turbine client imports
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
-- need localPlayer to test for session play
localPlayer=Turbine.Gameplay.LocalPlayer.GetInstance();
charName=localPlayer:GetName();
charVaultName=charName.." (Vault)";
-- mmediately test for session play BEFORE importing any other files so that we can safely bail without loading any data and without any handlers
if string.sub(charName,1,1)=="~" then
	-- do not load if the character is a session play character
	Turbine.Shell.WriteLine("Alt Inventory does not support session play characters.");
	return
end

string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end
replaceStr={[128]="A",[129]="A",[130]="A",[131]="A",[132]="A",[133]="A",[134]="AE",[135]="C",[136]="E",[137]="E",[138]="E",[139]="E",[140]="I",[141]="I",[142]="I",[143]="I",[144]="",[145]="N",[146]="O",[147]="O",[148]="O",[149]="O",[150]="O",[151]="",[152]="O",[153]="U",[154]="U",[155]="U",[156]="U",[157]="Y",[158]="Y",[159]="sz",[160]="a",[161]="a",[162]="a",[163]="a",[164]="a",[165]="a",[166]="ae",[167]="c",[168]="e",[169]="e",[170]="e",[171]="e",[172]="i",[173]="i",[174]="i",[175]="i",[176]="o",[177]="n",[178]="o",[179]="o",[180]="o",[181]="o",[182]="o",[183]="",[184]="o",[185]="u",[186]="u",[187]="u",[188]="u",[189]="y",[190]="y",[191]="y"}
string.stripaccent=function(str)
	local ret = "";
	local replace=false
	for i,v in ipairs({str:byte(1,-1)}) do
		if replace then
			replace=false
			if replaceStr[v]~=nil then
				ret=ret..replaceStr[v]
			end
		else
			if (v==195) then
				replace=true
			else
				ret = ret .. string.char(v);
			end
		end
	end
	return ret;
end

import (importPath.."Class");
import (importPath.."VindarPatch");
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import (importPath.."ColorPicker");
import (importPath.."Strings");
import (importPath.."Table");
import (importPath.."RadioButtonGroup");
import (importPath.."DropDownList");
import (importPath.."DebugWindow");
import (importPath.."FontSupport");

language=1;
euroFormat=(tonumber("1,000")==1);
if euroFormat then
	function euroNormalize(value)
		return tonumber((string.gsub(value,"%.",",")));
	end
else
	function euroNormalize(value)
		return tonumber((string.gsub(value,",",".")));
	end
end

locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

setupVisible=false;
inventoryVisible=false;
minimalVisible=false;
iconVisible=false;
hudVisible=true; -- assume hud is visible when loading plugins -- I hate this assumption, but Turbine has failed to provide a legitimate means of testing the hud state
moveToggle=false;
displayWidth=Turbine.UI.Display:GetWidth();
displayHeight=Turbine.UI.Display:GetHeight();

CharList=PatchDataLoad(Turbine.DataScope.Server, "AltInventoryCharList");
if CharList==nil then
	CharList={};
	CharList["Shared Storage"]={};
end
if CharList[charName]==nil then
	CharList[charName]={};
end
if CharList[charVaultName]==nil then
	CharList[charVaultName]={};
end

Settings=PatchDataLoad(Turbine.DataScope.Character, "AltInventorySettings");
if Settings==nil then
	Settings={}
end

if Settings.language==nil then Settings.language=language end
language=Settings.language;
if Settings.debug==nil then Settings.debug=false end;
Settings.debug=false; -- don't allow persistent debugging (to debug initialization this has to be temporarily changed to true)
if Settings.fontColor==nil then
	Settings.fontColor=Turbine.UI.Color(1,.9,.5)
else
	Settings.fontColor=Turbine.UI.Color(euroNormalize(Settings.fontColor.R),euroNormalize(Settings.fontColor.G),euroNormalize(Settings.fontColor.B))
end
if Settings.errorColor==nil then
	Settings.errorColor=Turbine.UI.Color(1,0,0)
else
	Settings.errorColor=Turbine.UI.Color(euroNormalize(Settings.errorColor.R),euroNormalize(Settings.errorColor.G),euroNormalize(Settings.errorColor.B))
end

if Settings.trimColor==nil then
	Settings.trimColor=Turbine.UI.Color(.4,.4,.5)
else
	Settings.trimColor=Turbine.UI.Color(euroNormalize(Settings.trimColor.R),euroNormalize(Settings.trimColor.G),euroNormalize(Settings.trimColor.B))
end
if Settings.backColor==nil then
	Settings.backColor=Turbine.UI.Color(.05,.05,.05)
else
	Settings.backColor=Turbine.UI.Color(euroNormalize(Settings.backColor.R),euroNormalize(Settings.backColor.G),euroNormalize(Settings.backColor.B))
end
if Settings.panelBackColor==nil then
	Settings.panelBackColor=Turbine.UI.Color(.25,.75,.45)
else
	Settings.panelBackColor=Turbine.UI.Color(euroNormalize(Settings.panelBackColor.R),euroNormalize(Settings.panelBackColor.G),euroNormalize(Settings.panelBackColor.B))
end
if Settings.headingsColor==nil then
	Settings.headingsColor=Turbine.UI.Color(1,1,1)
else
	Settings.headingsColor=Turbine.UI.Color(euroNormalize(Settings.headingsColor.R),euroNormalize(Settings.headingsColor.G),euroNormalize(Settings.headingsColor.B))
end
if Settings.listTextColor==nil then
	Settings.listTextColor=Turbine.UI.Color(1,1,1)
else
	Settings.listTextColor=Turbine.UI.Color(euroNormalize(Settings.listTextColor.R),euroNormalize(Settings.listTextColor.G),euroNormalize(Settings.listTextColor.B))
end
if Settings.critColor==nil then
	Settings.critColor=Turbine.UI.Color(.9,0,0)
else
	Settings.critColor=Turbine.UI.Color(euroNormalize(Settings.critColor.R),euroNormalize(Settings.critColor.G),euroNormalize(Settings.critColor.B))
end
if Settings.lowColor==nil then
	Settings.lowColor=Turbine.UI.Color(.8,.5,0)
else
	Settings.lowColor=Turbine.UI.Color(euroNormalize(Settings.lowColor.R),euroNormalize(Settings.lowColor.G),euroNormalize(Settings.lowColor.B))
end
if Settings.highColor==nil then
	Settings.highColor=Turbine.UI.Color(0,.8,0)
else
	Settings.highColor=Turbine.UI.Color(euroNormalize(Settings.highColor.R),euroNormalize(Settings.highColor.G),euroNormalize(Settings.highColor.B))
end

if Settings.fontFace==nil then Settings.fontFace=Turbine.UI.Lotro.Font.Verdana16 end
if Settings.opacity==nil then Settings.opacity=1 end
if Settings.lowQty==nil then Settings.lowQty=10 end
if Settings.critQty==nil then Settings.critQty=5 end
if Settings.highQty==nil then Settings.highQty=100 end
if Settings.lowShow==nil then Settings.lowShow=true end
if Settings.critShow==nil then Settings.critShow=true end
if Settings.normalShow==nil then Settings.normalShow=true end
if Settings.highShow==nil then Settings.highShow=true end
if Settings.showTotals==nil then Settings.showTotals=true end
if Settings.showCurrent==nil then Settings.showCurrent=true end
if Settings.outlineQtyText==nil then Settings.outlineQtyText=true end
if Settings.loadMinimized==nil then Settings.loadMinimized=false end
if Settings.useMinimalHeader==nil then Settings.useMinimalHeader=false end
if Settings.showIcon==nil then Settings.showIcon=true end
if Settings.bagSeparator then Settings.bagSeparator=false end
if Settings.panelViewMode==nil then Settings.panelViewMode=2 end
if Settings.useMiniIcon==nil then Settings.useMiniIcon=false end
if Settings.defaultToAllView==nil then Settings.defaultToAllView=false end

if Settings.panelWidth==nil then
	Settings.panelWidth=406/displayWidth;
else
	Settings.panelWidth=euroNormalize(Settings.panelWidth);
	if Settings.panelWidth+20/displayWidth>1 then Settings.panelWidth=1-20/displayWidth end
end
if Settings.panelHeight==nil then
	Settings.panelHeight=114/displayHeight;
else
	Settings.panelHeight=euroNormalize(Settings.panelHeight);
	if Settings.panelHeight+20/displayHeight>1 then Settings.panelHeight=1-20/displayHeight end
end
Settings.panelMinWidth=406/displayWidth;
Settings.panelMinHeight=114/displayHeight;

if Settings.panelLeft==nil then
	Settings.panelLeft=.5-Settings.panelWidth/2;
else
	Settings.panelLeft=euroNormalize(Settings.panelLeft);
	if Settings.panelLeft+Settings.panelWidth>1 then Settings.panelLeft=1-Settings.panelWidth end
end
if Settings.panelTop==nil then
	Settings.panelTop=.5-Settings.panelHeight/2;
else
	Settings.panelTop=euroNormalize(Settings.panelTop);
	if Settings.panelTop+Settings.panelHeight>1 then Settings.panelTop=1-Settings.panelHeight end
end
if Settings.replaceBags==nil then Settings.replaceBags=false end
if Settings.replaceBags then
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack1, false );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack2, false );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack3, false );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack4, false );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack5, false );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack6, false );
end
if Settings.totalsOnly==nil then Settings.totalsOnly=false end

colorGold=Turbine.UI.Color(.80,.60,.1);
colorSilver=Turbine.UI.Color(.75,.75,.75);
colorDarkGrey=Turbine.UI.Color(.1,.1,.1);
Tab1TabTrimColor=Turbine.UI.Color(.5,.5,.8);
Tab1TabBackColor=Turbine.UI.Color(.1,.1,.15);
Tab2TabTrimColor=Turbine.UI.Color(.8,.5,.5);
Tab2TabBackColor=Turbine.UI.Color(.15,.1,.1);
Tab3TabTrimColor=Turbine.UI.Color(.8,.8,.85);
Tab3TabBackColor=Turbine.UI.Color(.15,.15,.15);
errorColor=Turbine.UI.Color(1,.2,.2);
successColor=Turbine.UI.Color(.2,1,.2);
fontMetric=FontMetric();
fontMetric:SetFont(Settings.fontFace);

-- the Turbine Quality enumeration is NOT ordered correctly :(
function GetQualityCode(quality)
	local tmpVal=quality
	if tmpVal==2 then
		tmpVal=3
	elseif tmpVal==3 then
		tmpVal=2
	end
	return tmpVal;
end
function GetQualityColor(qualityCode)
	local tmpColor=Turbine.UI.Color.White
	if qualityCode==1 then
		tmpColor=Turbine.UI.Color.Gold
	elseif qualityCode==2 then
		tmpColor=Turbine.UI.Color.Aqua
	elseif qualityCode==3 then
		tmpColor=Turbine.UI.Color.DarkMagenta
	elseif qualityCode==4 then
		tmpColor=Turbine.UI.Color.Yellow
	end
	return tmpColor		
end
function GetCategoryCode(category)
	-- for now just return the Turbine category
	return category;
end

-- This is the callback mechanism provided by Pengoros, slightly modified to guarantee uniqueness
function AddCallback(object, event, callback)
	if (object[event] == nil) then
		object[event] = callback;
	else
		if (type(object[event]) == "table") then
			local exists=false;
			local k,v;
			for k,v in ipairs(object[event]) do
				if v==callback then
					exists=true;
					break;
				end
			end
			if not exists then
				table.insert(object[event], callback);
			end
		else
			if object[event]~=callback then
				object[event] = {object[event], callback};
			end
		end
	end
	return callback;
end

-- safely remove a callback without clobbering any extras
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

import (importPath.."ItemCategory");
import (importPath.."InventoryWindow");
import (importPath.."IconWindow");

function UnloadPlugin()
	SaveData();

	-- have to remove all shell commands before unloading or we get a crash to desktop :(
	if setupWindow~=nil then
		Turbine.Shell.RemoveCommand(setupWindow.shellCmd)
	end

	-- safely remove all callbacks
	local index;
	for index=1,backPack:GetSize() do
		if backpackImage[index]~=nil and backpackImage[index].Item~=nil then
			if backpackImage[index].Item.QuantityChanged~=nil then
				RemoveCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged)
			end
		end
	end
	for index=1,#sharedStorageImage do
		if sharedStorageImage[index].Item~=nil then
			if sharedStorageImage[index].Item.QuantityChanged~=nil then
				RemoveCallback(sharedStorageImage[index].Item,"QuantityChanged",sharedStorageQuantityChanged)
				RemoveCallback(sharedStorageImage[index].Item,"ChestChanged",sharedStorageChestChanged)
			end
		end
	end
	for index=1,#vaultImage do
		if vaultImage[index].Item~=nil then
			if vaultImage[index].Item.QuantityChanged~=nil then
				RemoveCallback(vaultImage[index].Item,"QuantityChanged",vaultQuantityChanged)
				RemoveCallback(vaultImage[index].Item,"ChestChanged",vaultChestChanged)
			end
		end
	end

	RemoveCallback(backPack,"ItemAdded",itemAdded)
	RemoveCallback(backPack,"ItemRemoved",itemRemoved)
	RemoveCallback(backPack,"ItemMoved",itemMoved)
	RemoveCallback(sharedStorage,"ItemAdded",sharedStorageItemAdded);
	RemoveCallback(sharedStorage,"ItemRemoved",sharedStorageItemRemoved);
	RemoveCallback(sharedStorage,"IsAvailableChanged",refreshSharedStorage);
	RemoveCallback(sharedStorage,"ItemsRefreshed",refreshSharedStorage);
	RemoveCallback(vault,"ItemAdded",vaultItemAdded);
	RemoveCallback(vault,"ItemRemoved",vaultItemRemoved);
	RemoveCallback(vault,"IsAvailableChanged",refreshVault);
	RemoveCallback(vault,"ItemsRefreshed",refreshVault);

	-- reenable built-in bags
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack1, true );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack2, true );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack3, true );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack4, true );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack5, true );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack6, true );
end
function SaveData()
	CharList[charName].VaultChestNames={};
	for index=1,vault:GetChestCount() do
		CharList[charName].VaultChestNames[index]=vault:GetChestName(index-1);
	end
	PatchDataSave(Turbine.DataScope.Server,"AltInventoryCharList",CharList);

	-- save the tooltip settings
	local tmpIndex;
	Settings.language=language;
	Settings.version=Plugins.AltInventory:GetVersion();
	Settings.SWTop=setupWindow:GetTop();
	Settings.SWLeft=setupWindow:GetLeft();
	local parent=inventoryPanel:GetParent();
	Settings.panelLeft=(parent:GetLeft()+inventoryPanel:GetLeft())/displayWidth;
	Settings.panelTop=(parent:GetTop()+inventoryPanel:GetTop())/displayHeight;
	Settings.iconTop=iconWindow:GetTop()/displayHeight;
	Settings.iconLeft=iconWindow:GetLeft()/displayWidth;
	if Settings.snapShot==nil then Settings.snapShot=0 end
	Settings.snapShot=Settings.snapShot+1
	if Settings.snapShot>10 then Settings.snapShot=1 end
	PatchDataSave(Turbine.DataScope.Character, "AltInventorySettings", Settings);

	if accountItemQty==nil then accountItemQty={} end

	-- remove the listItem link and itemInfo values and any entries with qty=0 since they are irrelevant
	for k,v in pairs(accountItemQty) do
		local total=0;
		v.ListItem=nil;
		v.ItemInfo=nil;
		for char,qty in pairs(v.Qty) do
			if (type(qty)=="table" and qty.Subtotal==0) or (type(qty)=="number" and qty==0) then
				v.Qty[char]=nil;
			else
				if char==charName then
					local bagTotal={};
					local subtotal=0;
					-- rollup the current char bag inventory - only need to store the qty per bag, not each stack
					for slot, slotQty in pairs(qty) do
						if slot~="Subtotal" then
							local bagIndex=math.floor((slot+14)/15)
							if bagTotal[bagIndex]==nil then
								bagTotal[bagIndex]=slotQty
							else
								bagTotal[bagIndex]=bagTotal[bagIndex]+slotQty
							end
								subtotal=subtotal+slotQty
						end
					end
					if subtotal==0 then
						accountItemQty[k].Qty[charName]=nil;
					else
						accountItemQty[k].Qty[charName]={["Subtotal"]=subtotal}
						for bag,bagQty in pairs(bagTotal) do
							accountItemQty[k].Qty[charName][bag]=bagQty;
						end
					end
				end
				if type(qty)=="table" then
					total=total+qty.Subtotal;
				else
					total=total+qty;
				end
			end
		end
		if total==0 then
			accountItemQty[k]=nil;
		end
	end

	PatchDataSave(Turbine.DataScope.Server, "AltInventoryData", accountItemQty);
	PatchDataSave(Turbine.DataScope.Server, "AltInventoryData_SnapShot"..tostring(Settings.snapShot), accountItemQty);
end

setupWindow = Turbine.UI.Lotro.Window();
setupWindow.Loaded=false;
setupWindow:SetZOrder(1);
local initHeight=410;
local initWidth=445;
if Settings.SWTop==nil then Settings.SWTop=(displayHeight - initHeight) / 2 end
if Settings.SWLeft==nil then Settings.SWLeft=(displayWidth - initWidth) / 2 end

if initHeight>displayHeight then initHeight=displayHeight end
if initHeight+Settings.SWTop>displayHeight then Settings.SWTop=displayHeight-initHeight end
if initWidth>displayWidth then initWidth=displayWidth end
if initWidth+Settings.SWLeft>displayWidth then Settings.SWLeft=displayWidth-initWidth end

setupWindow:SetPosition( Settings.SWLeft, Settings.SWTop );
setupWindow:SetHeight(initHeight);
setupWindow:SetWidth(initWidth);
setupWindow:SetOpacity(Settings.opacity);
setupWindow:SetVisible(false);

setupWindow.TabTrim=Turbine.UI.Label();
setupWindow.TabTrim:SetParent(setupWindow);
setupWindow.TabTrim:SetSize(setupWindow:GetWidth()-20,setupWindow:GetHeight()-105);
setupWindow.TabTrim:SetPosition(10,66);
setupWindow.TabTrim:SetBackColor(Tab1TabTrimColor);

setupWindow.TabBack=Turbine.UI.Label();
setupWindow.TabBack:SetParent(setupWindow.TabTrim);
setupWindow.TabBack:SetSize(setupWindow.TabTrim:GetWidth()-2,setupWindow.TabTrim:GetHeight()-2);
setupWindow.TabBack:SetPosition(1,1);
setupWindow.TabBack:SetBackColor(Tab1TabTrimColor);

-- list tab
setupWindow.Tab1Tab=Turbine.UI.Label();
setupWindow.Tab1Tab:SetParent(setupWindow.TabBack);
setupWindow.Tab1Tab:SetPosition(1,1);
setupWindow.Tab1Tab:SetSize(setupWindow.TabBack:GetWidth()-2,setupWindow.TabBack:GetHeight()-2);
setupWindow.Tab1Tab:SetBackColor(Tab1TabBackColor);

-- defaults tab
setupWindow.Tab2Tab=Turbine.UI.Label();
setupWindow.Tab2Tab:SetParent(setupWindow.TabBack);
setupWindow.Tab2Tab:SetPosition(1,1);
setupWindow.Tab2Tab:SetSize(setupWindow.TabBack:GetWidth()-2,setupWindow.TabBack:GetHeight()-2);
setupWindow.Tab2Tab:SetBackColor(Tab2TabBackColor);
setupWindow.Tab2Tab:SetVisible(false);

setupWindow.Tab3Tab=Turbine.UI.Label();
setupWindow.Tab3Tab:SetParent(setupWindow.TabBack);
setupWindow.Tab3Tab:SetPosition(1,1);
setupWindow.Tab3Tab:SetSize(setupWindow.TabBack:GetWidth()-2,setupWindow.TabBack:GetHeight()-2);
setupWindow.Tab3Tab:SetBackColor(Tab3TabBackColor);
setupWindow.Tab3Tab:SetVisible(false);

-- tab buttons
setupWindow.Tab1Button=Turbine.UI.Control()
setupWindow.Tab1Button:SetParent(setupWindow)
setupWindow.Tab1Button:SetPosition(10,42);
setupWindow.Tab1Button:SetSize(210,26);
setupWindow.Tab1Button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
setupWindow.Tab1Button:SetBackground(resourcePath.."TabBack_up.tga");
setupWindow.Tab1Button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
setupWindow.Tab1Button:SetBackColor(Tab1TabTrimColor)
setupWindow.Tab1Text=Turbine.UI.Label()
setupWindow.Tab1Text:SetParent(setupWindow.Tab1Button)
setupWindow.Tab1Text:SetPosition(4,4);
setupWindow.Tab1Text:SetSize(202,17);
setupWindow.Tab1Text:SetBackColor(Tab1TabBackColor);
setupWindow.Tab1Text:SetFont(Settings.fontFace);
setupWindow.Tab1Text:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
setupWindow.Tab1Text:SetText(Resource[language][24]);
setupWindow.Tab1Text:SetMouseVisible(false);
setupWindow.Tab1Button.MouseClick=function()
	setupWindow.Tab1Tab:SetVisible(true);
	setupWindow.TabTrim:SetBackColor(Tab1TabTrimColor);
	setupWindow.TabBack:SetBackColor(Tab1TabTrimColor);
	setupWindow.Tab2Tab:SetVisible(false);
	setupWindow.Tab3Tab:SetVisible(false);
	setupWindow.Tab1Button:SetBackground(resourcePath.."TabBack_up.tga");
	setupWindow.Tab2Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab3Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab1Button:SetTop(42);
	setupWindow.Tab1Button:SetHeight(26);
	setupWindow.Tab2Button:SetTop(46);
	setupWindow.Tab2Button:SetHeight(20);
	setupWindow.Tab3Button:SetTop(46);
	setupWindow.Tab3Button:SetHeight(20);
end
setupWindow.Tab2Button=Turbine.UI.Control()
setupWindow.Tab2Button:SetParent(setupWindow)
setupWindow.Tab2Button:SetPosition(225,46);
setupWindow.Tab2Button:SetSize(210,20);
setupWindow.Tab2Button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
setupWindow.Tab2Button:SetBackground(resourcePath.."TabBack_down.tga");
setupWindow.Tab2Button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
setupWindow.Tab2Button:SetBackColor(Tab2TabTrimColor)
setupWindow.Tab2Text=Turbine.UI.Label()
setupWindow.Tab2Text:SetParent(setupWindow.Tab2Button)
setupWindow.Tab2Text:SetPosition(4,4);
setupWindow.Tab2Text:SetSize(202,18);
--setupWindow.Tab2Text:SetBackColor(Tab2TabBackColor);
setupWindow.Tab2Text:SetFont(Settings.fontFace);
setupWindow.Tab2Text:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
setupWindow.Tab2Text:SetText(Resource[language][25]);
setupWindow.Tab2Text:SetMouseVisible(false);
setupWindow.Tab2Button.MouseClick=function()
	setupWindow.Tab2Tab:SetVisible(true);
	setupWindow.TabTrim:SetBackColor(Tab2TabTrimColor);
	setupWindow.TabBack:SetBackColor(Tab2TabTrimColor);
	setupWindow.Tab1Tab:SetVisible(false);
	setupWindow.Tab3Tab:SetVisible(false);
	setupWindow.Tab1Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab2Button:SetBackground(resourcePath.."TabBack_up.tga");
	setupWindow.Tab3Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab2Button:SetTop(42);
	setupWindow.Tab2Button:SetHeight(26);
	setupWindow.Tab1Button:SetTop(46);
	setupWindow.Tab1Button:SetHeight(20);
	setupWindow.Tab3Button:SetTop(46);
	setupWindow.Tab3Button:SetHeight(20);
end

setupWindow.Tab3Button=Turbine.UI.Control()
setupWindow.Tab3Button:SetParent(setupWindow)
setupWindow.Tab3Button:SetPosition(252,46);
setupWindow.Tab3Button:SetSize(210,20);
setupWindow.Tab3Button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
setupWindow.Tab3Button:SetBackground(resourcePath.."TabBack_down.tga");
setupWindow.Tab3Button:SetBackColor(Tab3TabTrimColor)
setupWindow.Tab3Button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
setupWindow.Tab3Button:SetVisible(false);

setupWindow.Tab3Text=Turbine.UI.Label()
setupWindow.Tab3Text:SetParent(setupWindow.Tab3Button)
setupWindow.Tab3Text:SetPosition(4,4);
setupWindow.Tab3Text:SetSize(202,15);
--setupWindow.Tab3Text:SetBackColor(Tab3TabBackColor);
setupWindow.Tab3Text:SetFont(Settings.fontFace);
setupWindow.Tab3Text:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
setupWindow.Tab3Text:SetText(Resource[language][26]);
setupWindow.Tab3Text:SetMouseVisible(false);
setupWindow.Tab3Button.MouseClick=function()
	setupWindow.Tab3Tab:SetVisible(true);
	setupWindow.TabTrim:SetBackColor(Tab3TabTrimColor);
	setupWindow.TabBack:SetBackColor(Tab3TabTrimColor);
	setupWindow.Tab1Tab:SetVisible(false);
	setupWindow.Tab2Tab:SetVisible(false);
	setupWindow.Tab1Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab2Button:SetBackground(resourcePath.."TabBack_down.tga");
	setupWindow.Tab3Button:SetBackground(resourcePath.."TabBack_up.tga");
	setupWindow.Tab3Button:SetTop(42);
	setupWindow.Tab3Button:SetHeight(26);
	setupWindow.Tab1Button:SetTop(46);
	setupWindow.Tab1Button:SetHeight(20);
	setupWindow.Tab2Button:SetTop(46);
	setupWindow.Tab2Button:SetHeight(20);
end

-- tab1 controls
setupWindow.LanguageCaption=Turbine.UI.Label();
setupWindow.LanguageCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.LanguageCaption:SetSize(120,20);
setupWindow.LanguageCaption:SetPosition(5,10);
setupWindow.LanguageCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.LanguageCaption:SetForeColor(Settings.fontColor);
setupWindow.LanguageCaption:SetText(Resource[language][28]..":");
setupWindow.LanguageList=DropDownList();
setupWindow.LanguageList:SetParent(setupWindow.Tab1Tab);
setupWindow.LanguageList:SetPosition(setupWindow.LanguageCaption:GetLeft()+setupWindow.LanguageCaption:GetWidth()+5,setupWindow.LanguageCaption:GetTop());
setupWindow.LanguageList:SetSize(setupWindow.Tab1Tab:GetWidth()-5-setupWindow.LanguageList:GetLeft(),20)
setupWindow.LanguageList:SetBackColor(Settings.backColor);
setupWindow.LanguageList:SetCurrentBackColor(Settings.backColor);
setupWindow.LanguageList:SetBorderColor(Settings.trimColor);
setupWindow.LanguageList:SetZOrder(1);
setupWindow.LanguageList:SetTextColor(Settings.listTextColor);
for k,v in pairs(Resource) do
	setupWindow.LanguageList:AddItem(v[1],k);
end
setupWindow.LanguageList:SetSelectedIndex(language);

setupWindow.TrimCaption=Turbine.UI.Label();
setupWindow.TrimCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.TrimCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.TrimCaption:SetPosition(setupWindow.LanguageCaption:GetLeft(),setupWindow.LanguageCaption:GetTop()+setupWindow.LanguageCaption:GetHeight()+10);
setupWindow.TrimCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.TrimCaption:SetForeColor(Settings.fontColor);
setupWindow.TrimCaption:SetText(Resource[language][29]..":");
setupWindow.TrimColor=ColorPicker();
setupWindow.TrimColor:SetParent(setupWindow.Tab1Tab);
setupWindow.TrimColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.TrimCaption:GetTop());
setupWindow.BackCaption=Turbine.UI.Label();
setupWindow.BackCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.BackCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.BackCaption:SetPosition(setupWindow.TrimCaption:GetLeft(),setupWindow.TrimCaption:GetTop()+setupWindow.TrimCaption:GetHeight()+10);
setupWindow.BackCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.BackCaption:SetForeColor(Settings.fontColor);
setupWindow.BackCaption:SetText(Resource[language][30]..":");
setupWindow.BackColor=ColorPicker();
setupWindow.BackColor:SetParent(setupWindow.Tab1Tab);
setupWindow.BackColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.BackCaption:GetTop());
setupWindow.TextCaption=Turbine.UI.Label();
setupWindow.TextCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.TextCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.TextCaption:SetPosition(setupWindow.BackCaption:GetLeft(),setupWindow.BackCaption:GetTop()+setupWindow.BackCaption:GetHeight()+10);
setupWindow.TextCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.TextCaption:SetForeColor(Settings.fontColor);
setupWindow.TextCaption:SetText(Resource[language][31]..":");
setupWindow.TextColor=ColorPicker();
setupWindow.TextColor:SetParent(setupWindow.Tab1Tab);
setupWindow.TextColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.TextCaption:GetTop());
setupWindow.HeadingsCaption=Turbine.UI.Label();
setupWindow.HeadingsCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.HeadingsCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.HeadingsCaption:SetPosition(setupWindow.BackCaption:GetLeft(),setupWindow.TextCaption:GetTop()+setupWindow.TextCaption:GetHeight()+10);
setupWindow.HeadingsCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.HeadingsCaption:SetForeColor(Settings.fontColor);
setupWindow.HeadingsCaption:SetText(Resource[language][33]..":");
setupWindow.HeadingsColor=ColorPicker();
setupWindow.HeadingsColor:SetParent(setupWindow.Tab1Tab);
setupWindow.HeadingsColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.HeadingsCaption:GetTop());
setupWindow.ListCaption=Turbine.UI.Label();
setupWindow.ListCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.ListCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.ListCaption:SetPosition(setupWindow.BackCaption:GetLeft(),setupWindow.HeadingsCaption:GetTop()+setupWindow.HeadingsCaption:GetHeight()+10);
setupWindow.ListCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.ListCaption:SetForeColor(Settings.fontColor);
setupWindow.ListCaption:SetText(Resource[language][34]..":");
setupWindow.ListColor=ColorPicker();
setupWindow.ListColor:SetParent(setupWindow.Tab1Tab);
setupWindow.ListColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.ListCaption:GetTop());
setupWindow.PanelCaption=Turbine.UI.Label();
setupWindow.PanelCaption:SetParent(setupWindow.Tab1Tab);
setupWindow.PanelCaption:SetSize(setupWindow.LanguageCaption:GetSize());
setupWindow.PanelCaption:SetPosition(setupWindow.BackCaption:GetLeft(),setupWindow.ListCaption:GetTop()+setupWindow.ListCaption:GetHeight()+10);
setupWindow.PanelCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.PanelCaption:SetForeColor(Settings.fontColor);
setupWindow.PanelCaption:SetText(Resource[language][35]..":");
setupWindow.PanelColor=ColorPicker();
setupWindow.PanelColor:SetParent(setupWindow.Tab1Tab);
setupWindow.PanelColor:SetPosition(setupWindow.LanguageList:GetLeft(),setupWindow.PanelCaption:GetTop());
setupWindow.DefaultColors=Turbine.UI.Lotro.Button();
setupWindow.DefaultColors:SetParent(setupWindow.Tab1Tab);
setupWindow.DefaultColors:SetSize(200,20);
setupWindow.DefaultColors:SetPosition((setupWindow.Tab1Tab:GetWidth()-setupWindow.DefaultColors:GetWidth())/2,setupWindow.PanelCaption:GetTop()+setupWindow.PanelCaption:GetHeight()+10)
setupWindow.DefaultColors:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.DefaultColors:SetText(Resource[language][32]);
setupWindow.DefaultColors.Click=function()
	setupWindow.TrimColor:SetColor(Turbine.UI.Color(.4,.4,.5));
	setupWindow.BackColor:SetColor(Turbine.UI.Color(.05,.05,.05));
	setupWindow.TextColor:SetColor(Turbine.UI.Color(1,.9,.5));
	setupWindow.HeadingsColor:SetColor(Turbine.UI.Color(1,1,1));
	setupWindow.ListColor:SetColor(Turbine.UI.Color(1,1,1));
	setupWindow.PanelColor:SetColor(Turbine.UI.Color(.25,.75,.45));
end

-- tab2 controls
setupWindow.replaceBags=Turbine.UI.Lotro.CheckBox();
setupWindow.replaceBags:SetParent(setupWindow.Tab2Tab);
setupWindow.replaceBags:SetSize(setupWindow.Tab2Tab:GetWidth()-20,20);
setupWindow.replaceBags:SetPosition(10,10);
setupWindow.replaceBags:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.replaceBags:SetForeColor(Settings.fontColor);
setupWindow.replaceBags:SetText(Resource[language][38]);

setupWindow.totalsOnly=Turbine.UI.Lotro.CheckBox();
setupWindow.totalsOnly:SetParent(setupWindow.Tab2Tab);
setupWindow.totalsOnly:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.totalsOnly:SetPosition(setupWindow.replaceBags:GetLeft(),setupWindow.replaceBags:GetTop()+setupWindow.replaceBags:GetHeight()+10);
setupWindow.totalsOnly:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.totalsOnly:SetForeColor(Settings.fontColor);
setupWindow.totalsOnly:SetText(Resource[language][53]);

setupWindow.loadMinimized=Turbine.UI.Lotro.CheckBox();
setupWindow.loadMinimized:SetParent(setupWindow.Tab2Tab);
setupWindow.loadMinimized:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.loadMinimized:SetPosition(setupWindow.replaceBags:GetLeft(),setupWindow.totalsOnly:GetTop()+setupWindow.totalsOnly:GetHeight()+10);
setupWindow.loadMinimized:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.loadMinimized:SetForeColor(Settings.fontColor);
setupWindow.loadMinimized:SetText(Resource[language][39]);

setupWindow.showIcon=Turbine.UI.Lotro.CheckBox();
setupWindow.showIcon:SetParent(setupWindow.Tab2Tab);
setupWindow.showIcon:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.showIcon:SetPosition(setupWindow.loadMinimized:GetLeft(),setupWindow.loadMinimized:GetTop()+setupWindow.loadMinimized:GetHeight()+10);
setupWindow.showIcon:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.showIcon:SetForeColor(Settings.fontColor);
setupWindow.showIcon:SetText(Resource[language][40]);

setupWindow.iconLeftCaption=Turbine.UI.Label();
setupWindow.iconLeftCaption:SetParent(setupWindow.Tab2Tab);
setupWindow.iconLeftCaption:SetSize(setupWindow.Tab2Tab:GetWidth()/4-10,20);
setupWindow.iconLeftCaption:SetPosition(10,setupWindow.showIcon:GetTop()+setupWindow.showIcon:GetHeight()+10);
setupWindow.iconLeftCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.iconLeftCaption:SetForeColor(Settings.fontColor);
setupWindow.iconLeftCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
setupWindow.iconLeftCaption:SetText(Resource[language][41]..":");
setupWindow.iconLeft=Turbine.UI.Lotro.TextBox();
setupWindow.iconLeft:SetParent(setupWindow.Tab2Tab);
setupWindow.iconLeft:SetSize(setupWindow.Tab2Tab:GetWidth()/4-10,20);
setupWindow.iconLeft:SetPosition(setupWindow.Tab2Tab:GetWidth()/4+5,setupWindow.iconLeftCaption:GetTop());
setupWindow.iconLeft:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.iconLeft:SetText("");

setupWindow.iconTopCaption=Turbine.UI.Label();
setupWindow.iconTopCaption:SetParent(setupWindow.Tab2Tab);
setupWindow.iconTopCaption:SetSize(setupWindow.iconLeftCaption:GetWidth(),20);
setupWindow.iconTopCaption:SetPosition(setupWindow.Tab2Tab:GetWidth()/2+5,setupWindow.iconLeftCaption:GetTop());
setupWindow.iconTopCaption:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.iconTopCaption:SetForeColor(Settings.fontColor);
setupWindow.iconTopCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
setupWindow.iconTopCaption:SetText(Resource[language][42]..":");
setupWindow.iconTop=Turbine.UI.Lotro.TextBox();
setupWindow.iconTop:SetParent(setupWindow.Tab2Tab);
setupWindow.iconTop:SetSize(setupWindow.iconLeft:GetWidth(),20);
setupWindow.iconTop:SetPosition(setupWindow.Tab2Tab:GetWidth()*3/4,setupWindow.iconTopCaption:GetTop());
setupWindow.iconTop:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.iconTop:SetText("");

setupWindow.miniIcon=Turbine.UI.Lotro.CheckBox();
setupWindow.miniIcon:SetParent(setupWindow.Tab2Tab);
setupWindow.miniIcon:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.miniIcon:SetPosition(setupWindow.loadMinimized:GetLeft(),setupWindow.iconTop:GetTop()+setupWindow.iconTop:GetHeight()+10);
setupWindow.miniIcon:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.miniIcon:SetForeColor(Settings.fontColor);
setupWindow.miniIcon:SetText(Resource[language][52]);

setupWindow.useMinimalHeader=Turbine.UI.Lotro.CheckBox();
setupWindow.useMinimalHeader:SetParent(setupWindow.Tab2Tab);
setupWindow.useMinimalHeader:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.useMinimalHeader:SetPosition(setupWindow.replaceBags:GetLeft(),setupWindow.miniIcon:GetTop()+setupWindow.miniIcon:GetHeight()+10);
setupWindow.useMinimalHeader:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.useMinimalHeader:SetForeColor(Settings.fontColor);
setupWindow.useMinimalHeader:SetText(Resource[language][43]);

setupWindow.defaultToAll=Turbine.UI.Lotro.CheckBox();
setupWindow.defaultToAll:SetParent(setupWindow.Tab2Tab);
setupWindow.defaultToAll:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.defaultToAll:SetPosition(setupWindow.replaceBags:GetLeft(),setupWindow.useMinimalHeader:GetTop()+setupWindow.useMinimalHeader:GetHeight()+10);
setupWindow.defaultToAll:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.defaultToAll:SetForeColor(Settings.fontColor);
setupWindow.defaultToAll:SetText(Resource[language][54]);
setupWindow.defaultToAll:SetVisible(true);

setupWindow.bagSeparator=Turbine.UI.Lotro.CheckBox();
setupWindow.bagSeparator:SetParent(setupWindow.Tab2Tab);
setupWindow.bagSeparator:SetSize(setupWindow.replaceBags:GetWidth(),20);
setupWindow.bagSeparator:SetPosition(setupWindow.replaceBags:GetLeft(),setupWindow.defaultToAll:GetTop()+setupWindow.defaultToAll:GetHeight()+10);
setupWindow.bagSeparator:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.bagSeparator:SetForeColor(Settings.fontColor);
setupWindow.bagSeparator:SetText(Resource[language][44]);
setupWindow.bagSeparator:SetVisible(false);

setupWindow.Refresh=function()
	setupWindow.LanguageList:SetSelectedIndex(language);
	setupWindow.TrimColor:SetColor(Settings.trimColor);
	setupWindow.BackColor:SetColor(Settings.backColor);
	setupWindow.TextColor:SetColor(Settings.fontColor);
	setupWindow.HeadingsColor:SetColor(Settings.headingsColor);
	setupWindow.ListColor:SetColor(Settings.listTextColor);
	setupWindow.PanelColor:SetColor(Settings.panelBackColor);
	setupWindow.replaceBags:SetChecked(Settings.replaceBags);
	setupWindow.totalsOnly:SetChecked(Settings.totalsOnly);
	setupWindow.loadMinimized:SetChecked(Settings.loadMinimized);
	setupWindow.showIcon:SetChecked(Settings.showIcon);
	setupWindow.iconLeft:SetText(iconWindow:GetLeft());
	setupWindow.iconTop:SetText(iconWindow:GetTop());
	setupWindow.useMinimalHeader:SetChecked(Settings.useMinimalHeader);
	setupWindow.miniIcon:SetChecked(Settings.useMiniIcon);
	setupWindow.defaultToAll:SetChecked(Settings.defaultToAllView);
	setupWindow.bagSeparator:SetChecked(Settings.bagSeparator);
	if Settings.useMinimalHeader then
		setupWindow.PanelTop=minimalWindow:GetTop()+inventoryPanel:GetTop();
		setupWindow.PanelLeft=minimalWindow:GetLeft()+inventoryPanel:GetLeft();
	else
		setupWindow.PanelTop=inventoryWindow:GetTop()+inventoryPanel:GetTop();
		setupWindow.PanelLeft=inventoryWindow:GetLeft()+inventoryPanel:GetLeft();
	end
end

setupWindow.Save=Turbine.UI.Lotro.Button();
setupWindow.Save:SetParent(setupWindow);
setupWindow.Save:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.Save:SetSize(100,20);
setupWindow.Save:SetPosition((setupWindow:GetWidth()-setupWindow.Save:GetWidth()*2)/3, setupWindow:GetHeight()-35);
setupWindow.Save:SetText(Resource[language][36]);
setupWindow.Save.Click=function()
	if language~=setupWindow.LanguageList:GetValue() then
		language=setupWindow.LanguageList:GetValue();

		-- set language
		setupWindow.Tab1Text:SetText(Resource[language][24]);
		setupWindow.Tab2Text:SetText(Resource[language][25]);
		setupWindow.Tab3Text:SetText(Resource[language][26]);
		setupWindow.LanguageCaption:SetText(Resource[language][28]..":");
		setupWindow.TrimCaption:SetText(Resource[language][29]..":");
		setupWindow.BackCaption:SetText(Resource[language][30]..":");
		setupWindow.TextCaption:SetText(Resource[language][31]..":");
		setupWindow.HeadingsCaption:SetText(Resource[language][33]..":");
		setupWindow.ListCaption:SetText(Resource[language][34]..":");
		setupWindow.PanelCaption:SetText(Resource[language][35]..":");
		setupWindow.DefaultColors:SetText(Resource[language][32]);
		setupWindow.replaceBags:SetText(Resource[language][38]);
		setupWindow.loadMinimized:SetText(Resource[language][39]);
		setupWindow.showIcon:SetText(Resource[language][40]);
		setupWindow.iconLeftCaption:SetText(Resource[language][41]..":");
		setupWindow.iconTopCaption:SetText(Resource[language][42]..":");
		setupWindow.useMinimalHeader:SetText(Resource[language][43]);
		setupWindow.defaultToAll:SetText(Resource[language][54]);
		setupWindow.bagSeparator:SetText(Resource[language][44]);
		setupWindow.Save:SetText(Resource[language][36]);
		setupWindow.Cancel:SetText(Resource[language][37]);
		setupWindow:SetText(Resource[language][27]);
		if inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryWindow.CapacityDisplay:SetText(Resource[language][10]);
		elseif inventoryWindow.CharList:GetSelectedIndex()==2 then
			inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList["Shared Storage"].used).."/"..tostring(CharList["Shared Storage"].capacity)..")")
		else
			inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[inventoryWindow.CharList:GetText()].used).."/"..tostring(CharList[inventoryWindow.CharList:GetText()].capacity)..")")
		end
		inventoryWindow.CharList.ListData:GetItem(1):SetText(Resource[language][12]);
		if inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryWindow.CharList.CurrentValue:SetText(Resource[language][12]);
		end
		inventoryWindow.CharList.ListData:GetItem(2):SetText(Resource[language][9]);
		if inventoryWindow.CharList:GetSelectedIndex()==2 then
			inventoryWindow.CharList.CurrentValue:SetText(Resource[language][9]);
		end


		inventoryWindow:SetText("      "..Resource[language][2].."      ");
		inventoryWindow.CapacityEmpty:SetText(Resource[language][4]);
		inventoryWindow.CapacityFull:SetText(Resource[language][6]);
		inventoryWindow.CharCaption:SetText(Resource[language][13]..":");
		inventoryPanel.FilterCaption:SetText(Resource[language][11]..":");
		inventoryPanel.SearchCaption:SetText(Resource[language][14]..":");
		inventoryPanel.ShowAllButton:SetText(Resource[language][23]);
		inventoryPanel.SortCaption:SetText(Resource[language][15]..":");
		inventoryPanel.SortList.ListData:GetItem(1):SetText(Resource[language][17]);
		if inventoryPanel.SortList:GetSelectedIndex()==1 then
			inventoryPanel.SortList.CurrentValue:SetText(Resource[language][17])
		end
		inventoryPanel.SortList.ListData:GetItem(2):SetText(Resource[language][18]);
		if inventoryPanel.SortList:GetSelectedIndex()==2 then
			inventoryPanel.SortList.CurrentValue:SetText(Resource[language][18])
		end
		inventoryPanel.SortList.ListData:GetItem(3):SetText(Resource[language][19]);
		if inventoryPanel.SortList:GetSelectedIndex()==3 then
			inventoryPanel.SortList.CurrentValue:SetText(Resource[language][19])
		end
		inventoryPanel.SortList.ListData:GetItem(4):SetText(Resource[language][20]);
		if inventoryPanel.SortList:GetSelectedIndex()==4 then
			inventoryPanel.SortList.CurrentValue:SetText(Resource[language][20])
		end
		inventoryPanel.DisplayCaption:SetText(Resource[language][22]..":");

		-- filter list
		local tmpFilter=inventoryPanel.FilterList:GetValue();
		local newIndex=1;
		inventoryPanel.FilterList:ClearList();
		inventoryPanel.FilterList:AddItem(Resource[language][12],-1);
		for tmpIndex=1,#ItemCategory do
			if ItemCategory[tmpIndex][1]==Turbine.Gameplay.ItemCategory.Undefined then
				undefinedCategoryIndex=tmpIndex+1; -- need to keep track of where the "undefined" category winds up in the list
			end
			categorySortOrder[ItemCategory[tmpIndex][1]]=tmpIndex;
			if ItemCategory[tmpIndex][1]==tmpFilter then newIndex=tmpIndex+1 end
			inventoryPanel.FilterList:AddItem(ItemCategory[tmpIndex][2][language],ItemCategory[tmpIndex][1]);
		end
		inventoryPanel.FilterList:SetSelectedIndex(newIndex);
		inventoryPanel:Refresh();
		minimalWindow.CharText:SetText(Resource[language][13]..": "..inventoryWindow.CharList:GetText());
		minimalWindow.OptionsButton:SetText(Resource[language][48]);
	end
	-- trimColor
	Settings.trimColor=setupWindow.TrimColor:GetColor();
	setupWindow.LanguageList:SetBorderColor(Settings.trimColor);
	inventoryPanel.FilterList:SetBorderColor(Settings.trimColor);
	inventoryPanel.SortList:SetBorderColor(Settings.trimColor);

	-- backColor
	Settings.backColor=setupWindow.BackColor:GetColor();
	setupWindow.LanguageList:SetBackColor(Settings.backColor);
	setupWindow.LanguageList:SetCurrentBackColor(Settings.backColor);
	inventoryPanel.FilterList:SetBackColor(Settings.backColor);
	inventoryPanel.FilterList:SetCurrentBackColor(Settings.backColor);
	inventoryPanel.SortList:SetBackColor(Settings.backColor);
	inventoryPanel.SortList:SetCurrentBackColor(Settings.backColor);

	-- fontColor
	Settings.fontColor=setupWindow.TextColor:GetColor();
	setupWindow.LanguageCaption:SetForeColor(Settings.fontColor);
	setupWindow.TrimCaption:SetForeColor(Settings.fontColor);
	setupWindow.BackCaption:SetForeColor(Settings.fontColor);
	setupWindow.TextCaption:SetForeColor(Settings.fontColor);
	setupWindow.HeadingsCaption:SetForeColor(Settings.fontColor);
	setupWindow.ListCaption:SetForeColor(Settings.fontColor);
	setupWindow.PanelCaption:SetForeColor(Settings.fontColor);
	setupWindow.replaceBags:SetForeColor(Settings.fontColor);
	setupWindow.loadMinimized:SetForeColor(Settings.fontColor);
	setupWindow.showIcon:SetForeColor(Settings.fontColor);
	setupWindow.iconLeftCaption:SetForeColor(Settings.fontColor);
	setupWindow.iconTopCaption:SetForeColor(Settings.fontColor);
	setupWindow.useMinimalHeader:SetForeColor(Settings.fontColor);
	setupWindow.bagSeparator:SetForeColor(Settings.fontColor);
	inventoryWindow.CharCaption:SetForeColor(Settings.fontColor);
	inventoryPanel.FilterCaption:SetForeColor(Settings.fontColor);
	inventoryPanel.SearchCaption:SetForeColor(Settings.fontColor);
	inventoryPanel.SearchText:SetForeColor(Settings.fontColor);
	inventoryPanel.SortCaption:SetForeColor(Settings.fontColor);
	inventoryPanel.DisplayCaption:SetForeColor(Settings.fontColor);

	-- headingsColor
	Settings.headingsColor=setupWindow.HeadingsColor:GetColor();

	-- listTextColor
	Settings.listTextColor=setupWindow.ListColor:GetColor();
	setupWindow.LanguageList:SetTextColor(Settings.listTextColor);
	inventoryPanel.FilterList:SetTextColor(Settings.listTextColor);
	inventoryPanel.SortList:SetTextColor(Settings.listTextColor);

	-- panelBackColor
	Settings.panelBackColor=setupWindow.PanelColor:GetColor();
	inventoryWindow.TopPanelTiled:SetBackColor(Settings.panelBackColor);
	inventoryWindow.TopPanelLeft:SetBackColor(Settings.panelBackColor);
	inventoryWindow.TopPanelRight:SetBackColor(Settings.panelBackColor);
	inventoryWindow.BottomPanelTiled:SetBackColor(Settings.panelBackColor);
	inventoryWindow.BottomPanelLeft:SetBackColor(Settings.panelBackColor);
	inventoryWindow.BottomPanelRight:SetBackColor(Settings.panelBackColor);

	Settings.replaceBags=setupWindow.replaceBags:IsChecked();
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack1, not Settings.replaceBags );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack2, not Settings.replaceBags );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack3, not Settings.replaceBags );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack4, not Settings.replaceBags );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack5, not Settings.replaceBags );
	Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack6, not Settings.replaceBags );

	Settings.totalsOnly=setupWindow.totalsOnly:IsChecked();

	Settings.loadMinimized=setupWindow.loadMinimized:IsChecked();
	Settings.showIcon=setupWindow.showIcon:IsChecked();
	local val=tonumber(setupWindow.iconLeft:GetText());
	if val~=nil then
		val=math.floor(val);
		if val<0 then val=0 end
		if val>displayWidth-32 then
			val=displayWidth-32;
		end
		iconWindow:SetLeft(val);
	end
	val=tonumber(setupWindow.iconTop:GetText());
	if val~=nil then
		val=math.floor(val);
		if val<0 then val=0 end
		if val>displayHeight-32 then
			val=displayHeight-32;
		end
		iconWindow:SetTop(val);
	end
	Settings.useMinimalHeader=setupWindow.useMinimalHeader:IsChecked();
	if Settings.useMinimalHeader then
		inventoryPanel:SetParent(minimalWindow);
		inventoryPanel:SetPosition(minimalWindow.LeftMargin,minimalWindow.TopMargin);
		minimalWindow:SetPosition(setupWindow.PanelLeft-minimalWindow.LeftMargin,setupWindow.PanelTop-minimalWindow.TopMargin);
		minimalWindow:SetSize(inventoryPanel:GetWidth()+minimalWindow.LeftMargin*2,inventoryPanel:GetHeight()+minimalWindow.TopMargin+minimalWindow.BottomMargin);
	else
		inventoryPanel:SetParent(inventoryWindow);
		inventoryPanel:SetPosition(inventoryWindow.LeftMargin,inventoryWindow.TopMargin);
		inventoryWindow:SetPosition(setupWindow.PanelLeft-inventoryWindow.LeftMargin,setupWindow.PanelTop-inventoryWindow.TopMargin);
		inventoryWindow:SetSize(inventoryPanel:GetWidth()+inventoryWindow.LeftMargin*2,inventoryPanel:GetHeight()+inventoryWindow.TopMargin+inventoryWindow.BottomMargin);
		if inventoryWindow:GetLeft()<0 then inventoryWindow:SetLeft(0) end
		if inventoryWindow:GetTop()<0 then inventoryWindow:SetTop(0) end
		if inventoryWindow:GetHeight()>displayHeight then inventoryWindow:SetHeight(displayHeight) end
		if inventoryWindow:GetWidth()>displayWidth then inventoryWindow:SetWidth(displayWidth) end
	end

	Settings.useMiniIcon=setupWindow.miniIcon:IsChecked();
	if Settings.useMiniIcon then
		iconWindow.UpImage=resourcePath.."mini_icon.tga"
		iconWindow.HoverImage=resourcePath.."mini_hover.tga"
		iconWindow.MoveImage=resourcePath.."mini_drag.tga"
		iconWindow.FlashImage=resourcePath.."mini_border.tga"
	else
		iconWindow.UpImage=resourcePath.."bags_up.jpg"
		iconWindow.HoverImage=resourcePath.."bags_down.jpg"
		iconWindow.MoveImage=resourcePath.."bags_drag.jpg"
		iconWindow.FlashImage=resourcePath.."bags_border.tga"
	end
	iconWindow:SetBackground(iconWindow.UpImage);
	inventoryWindow:SetVisible(not Settings.useMinimalHeader);
	minimalWindow:SetVisible(Settings.useMinimalHeader);
	iconWindow:SetVisible(false); -- just incase the user closed the main window after opening the settings
	Settings.defaultToAllView=setupWindow.defaultToAll:IsChecked();
	if Settings.bagSeparator~=setupWindow.bagSeparator:IsChecked() then
		Settings.bagSeparator=setupWindow.bagSeparator:IsChecked();
		-- refresh view
	end

	inventoryPanel:Refresh();
	setupWindow:SetVisible(false);
end
setupWindow.Cancel=Turbine.UI.Lotro.Button();
setupWindow.Cancel:SetParent(setupWindow);
setupWindow.Cancel:SetFont(Turbine.UI.Lotro.Font.Verdana16);
setupWindow.Cancel:SetSize(100,20);
setupWindow.Cancel:SetPosition(setupWindow.Save:GetWidth()+setupWindow.Save:GetLeft()*2, setupWindow.Save:GetTop());
setupWindow.Cancel:SetText(Resource[language][37]);
setupWindow.Cancel.Click=function()
	setupWindow:Refresh();
	setupWindow:SetVisible(false);
end

setupWindow:SetText(Resource[language][27]);

setupWindow.KeyUp=function(sender, args)
	if bShowKeyDown==true then
		Turbine.Shell.WriteLine("KeyUp Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
	end
end

setupWindow.KeyDown=function(sender, args)
	if bShowKeyDown==true then
		Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
	end
	if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
		setupWindow:SetVisible( false );
		inventoryWindow:SetVisible(false);
		minimalWindow:SetVisible(false);
		iconWindow:SetVisible(Settings.showIcon);
	end
	if ( args.Action == 268435579 ) then -- reposition UI
		moveToggle=not moveToggle;
	end
	if ( args.Action == 268435635 ) then -- toggle HUD
		hudVisible=not hudVisible;
		if hudVisible then
			setupWindow:SetVisible(setupVisible);
			inventoryWindow:SetVisible(inventoryVisible);
			minimalWindow:SetVisible(minimalVisible);
			iconWindow:SetVisible(iconVisible and Settings.showIcon);
		else
			setupVisible=setupWindow:IsVisible();
			setupWindow:SetVisible(false);
			inventoryVisible=inventoryWindow:IsVisible();
			inventoryWindow:SetVisible(false);
			minimalVisible=minimalWindow:IsVisible();
			minimalWindow:SetVisible(false);
			iconVisible=iconWindow:IsVisible();
			iconWindow:SetVisible(false);
		end
	end
	if Settings.replaceBags and ( args.Action == Turbine.UI.Lotro.Action.ToggleBags or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag1 or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag2 or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag3 or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag4 or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag5 or
		args.Action == Turbine.UI.Lotro.Action.ToggleBag6 )
	then
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack1, false );
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack2, false );
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack3, false );
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack4, false );
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack5, false );
		Turbine.UI.Lotro.LotroUI.SetEnabled( Turbine.UI.Lotro.LotroUIElement.Backpack6, false );
		if Settings.useMinimalHeader then
			minimalWindow:SetVisible(not minimalWindow:IsVisible());
			if not minimalWindow:IsVisible() and Settings.showIcon then
				iconWindow:SetVisible(true)
			end
		else
			inventoryWindow:SetVisible(not inventoryWindow:IsVisible());
			if not inventoryWindow:IsVisible() and Settings.showIcon then
				iconWindow:SetVisible(true);
			end
		end
	end
end

setupWindow:SetWantsKeyEvents(true);
setupWindow:SetWantsUpdates(true)

setupWindow.shellCmd=Turbine.ShellCommand();
local numberOfCommandsRegistered = Turbine.Shell.AddCommand("AltInventory",setupWindow.shellCmd);

setupWindow.shellCmd.Execute = function(sender, cmd, args)
	if (string.lower(cmd) == "altinventory") then
		if (string.lower(args)=="setup") then
			setupWindow:SetVisible(true);
			setupWindow:Refresh();
		elseif (string.lower(args)=="show") then
			iconWindow:SetVisible(false);
			if Settings.useMinimalHeader then
				minimalWindow:SetVisible(true);
			else
				inventoryWindow:SetVisible(true);
			end
		elseif (string.lower(args)=="hide") then
			setupWindow:SetVisible(false);
			inventoryWindow:SetVisible(false);
			minimalWindow:SetVisible(false);
			if Settings.showIcon then
				iconWindow:SetVisible(true);
			end

		elseif (string.lower(args)=="toggle") then
			setupWindow:SetVisible(false);
			if inventoryWindow:IsVisible() or minimalWindow:IsVisible() then
				inventoryWindow:SetVisible(false);
				minimalWindow:SetVisible(false);
				if Settings.showIcon then
					iconWindow:SetVisible(true);
				end
			else
				iconWindow:SetVisible(false);
				if Settings.useMinimalHeader then
					minimalWindow:SetVisible(true);
				else
					inventoryWindow:SetVisible(true);
				end
			end

		elseif (string.lower(args)=="debug on") then
			Settings.debug=true;
			debugWindow:SetVisible(true);
			Turbine.Shell.WriteLine("Debug is now: "..tostring(Settings.debug))
		elseif (string.lower(args)=="debug off") then
			Settings.debug=false;
			Turbine.Shell.WriteLine("Debug is now: "..tostring(Settings.debug))
		elseif (string.lower(args)=="debug toggle") then
			Settings.debug=not Settings.debug;
			if debugWindow~=nil then
				debugWindow:SetVisible(Settings.debug);
			end
			Turbine.Shell.WriteLine("Debug is now: "..tostring(Settings.debug))
		elseif (string.lower(args)=="debug") then
			Turbine.Shell.WriteLine("Debug is: "..tostring(Settings.debug))
		else
			Turbine.Shell.WriteLine("usage:");
			Turbine.Shell.WriteLine("  /AltInventory setup");
			Turbine.Shell.WriteLine("       Displays the setup window.");
			Turbine.Shell.WriteLine("  /AltInventory show");
			Turbine.Shell.WriteLine("       Shows the main window.");
			Turbine.Shell.WriteLine("  /AltInventory hide");
			Turbine.Shell.WriteLine("       Hides the main window.");
		end
	end
end
setupWindow.shellCmd.GetHelp = function(sender, cmd)
	return("usage:\n  /AltInventory setup\n       Displays the setup window.\n  /AltInventory show\n       Shows the main window.\n  /AltInventory hide\n       Hides the main window.");
end

setupWindow.shellCmd.GetShortHelp = function(sender, cmd)
	return("AltInventory - usage: /AltInventory setup | show | hide");
end

setupWindow.Update=function( sender, args )
	if (Plugins.AltInventory ~= nil) and (not setupWindow.Loaded) then
		setupWindow:SetWantsUpdates(false)
		LoadData();
		if sharedStorage:IsAvailable() then
			refreshSharedStorage()
		end
		if vault:IsAvailable() then
			refreshVault()
		end
		setupWindow.Loaded=true;
		Turbine.Shell.WriteLine(Resource[language][2].." "..Plugins.AltInventory:GetVersion().." "..Resource[language][3]);
		Plugins.AltInventory.Unload = function(setupWindow,sender,args)
			UnloadPlugin();
		end
	end
end

backpackImage={}

backPack=localPlayer:GetBackpack()
CharList[charName].capacity=backPack:GetSize();

function quantityChanged(sender,args)
-- uses the brute force method of scanning the whole backpack. this will automatically resync any backpack quantity that gets out of sync due to odd sequences in event firing
-- I have to revisit this and determine whether the event firing order has been fixed in which case this can be significantly optimised to improve performance

	if sender~=nil and sender.GetItemInfo~=nil then
		local itemInfo=sender:GetItemInfo();
		if itemInfo~=nil then
			local name=sender:GetItemInfo():GetName();
			if name~=nil then
				local index=sender.Index
				local qty=sender:GetQuantity();
				if index==nil then
					if Settings.debug then
						Turbine.Shell.WriteLine("quantityChanged: index is nil")
					end
				else
					if accountItemQty[name]==nil then
						accountItemQty[name]={}
						accountItemQty[name].Total=0;
						accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
						accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
						accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
						accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
						accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
						accountItemQty[name].Qty={};
						accountItemQty[name].ListItem={};
						local found=false;
						for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
							if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
								if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charName then
									inventoryPanel.FilterList:ShowEntry(filterIndex);
								end
								found=true;
								break;
							end
						end
						if not found then
							accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
							if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charName then
								inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
							end
						end
					end
					if accountItemQty[name].ItemInfo==nil then
						accountItemQty[name].ItemInfo=itemInfo;
						if accountItemQty[name].ListItem[charName]~=nil then
							if Settings.totalsOnly then
								accountItemQty[name].ListItem[charName].ItemInfo:SetItemInfo(itemInfo);
								accountItemQty[name].ListItem[charName].IconBorder:SetVisible(false);
							else
								if accountItemQty[name].ListItem[charName][index]~=nil then
									accountItemQty[name].ListItem[charName][index].ItemInfo:SetItemInfo(itemInfo);
									accountItemQty[name].ListItem[charName][index].IconBorder:SetVisible(false);
								end
							end
						end
					end
					local oldQty=0;
					if accountItemQty[name].Qty[charName][index]~=nil then
						oldQty=accountItemQty[name].Qty[charName][index];
					end
					accountItemQty[name].Qty[charName][index]=qty;

					accountItemQty[name].Qty[charName].Subtotal=accountItemQty[name].Qty[charName].Subtotal-oldQty+qty;
					accountItemQty[name].Total=accountItemQty[name].Total-oldQty+qty;
					-- now update the display element
					if inventoryWindow.CharList:GetText()==charName then
						if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
						if Settings.totalsOnly then
							if accountItemQty[name].ListItem[charName]==nil then
								-- create a new entry, set height by filter and position it according to the sort
								local tmpEntry=ItemEntry(name,charName,accountItemQty[name].Qty[charName].Subtotal,itemInfo)
								if not Settings.totalsOnly then
									tmpEntry.Item:SetItem(backPack:GetItem(index))
									tmpEntry.Item.Index=index;
								end
								accountItemQty[name].ListItem[charName]=tmpEntry;
								inventoryPanel.ItemList:AddItem(tmpEntry)
								inventoryPanel.SortList:SelectedIndexChanged();
							else
								accountItemQty[name].ListItem[charName].Qty:SetText(accountItemQty[name].Qty[charName].Subtotal);
							end
						else
							if accountItemQty[name].ListItem[charName]==nil then accountItemQty[name].ListItem[charName]={} end
							if accountItemQty[name].ListItem[charName][index]==nil then
								-- create a new entry, set height by filter and position it according to the sort
								local tmpEntry=ItemEntry(name,charName,qty,itemInfo)
								if not Settings.totalsOnly then
									tmpEntry.Item:SetItem(backPack:GetItem(index))
									tmpEntry.Item.Index=index;
								end
								accountItemQty[name].ListItem[charName][index]=tmpEntry;
								inventoryPanel.ItemList:AddItem(tmpEntry)
								inventoryPanel.SortList:SelectedIndexChanged();
							else
								accountItemQty[name].ListItem[charName][index].Qty:SetText(qty);
							end
						end
					elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
						if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
						if accountItemQty[name].ListItem["ALL"]==nil then
							-- create a new entry, set height by filter and position it according to the sort
							local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
							accountItemQty[name].ListItem["ALL"]=tmpEntry;
							inventoryPanel.ItemList:AddItem(tmpEntry)
							inventoryPanel.SortList:SelectedIndexChanged();
						else
							accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
						end
					end
				end
			end
		end
	end
end


-- Shared Storage
sharedStorage=localPlayer:GetSharedStorage();
function sharedStorageChestChanged(sender)
	-- sender.Chest== old chest
	local oldChest=sender.Chest;
	local newChest=sender:GetChest();
	local qty=sender:GetQuantity();
	local name=sender:GetItemInfo():GetName();
	accountItemQty[name].Qty["Shared Storage"][oldChest]=accountItemQty[name].Qty["Shared Storage"][oldChest]-qty;
	if accountItemQty[name].Qty["Shared Storage"][oldChest]==0 then accountItemQty[name].Qty["Shared Storage"][oldChest]=nil end
	if accountItemQty[name].Qty["Shared Storage"][newChest]==nil then
		accountItemQty[name].Qty["Shared Storage"][newChest]=qty;
	else
		accountItemQty[name].Qty["Shared Storage"][newChest]=accountItemQty[name].Qty["Shared Storage"][newChest]+qty;
	end
	sender.Chest=newChest;
end
function sharedStorageQuantityChanged(sender,args)
	if Settings.debug then
		Turbine.Shell.WriteLine("sharedStorageQuantityChanged")
		Turbine.Shell.WriteLine("sender:"..tostring(sender))
	end
	local name=sender:GetItemInfo():GetName();
	local index;

	-- scan the shared storage, recalc the total for this name and then update accountItemQty
	local total=0;
	local containerQty={};
	for index=1,sharedStorage:GetCount() do
		local item=sharedStorage:GetItem(index);
		if item~=nil and item:GetItemInfo():GetName()==name then
			local tmpQty=item:GetQuantity();
			local bag=item:GetChest();
			total=total+tmpQty;
			if containerQty[bag]==nil then
				containerQty[bag]=tmpQty;
			else
				containerQty[bag]=containerQty[bag]+tmpQty;
			end
		end
	end
	local oldTotal=accountItemQty[name].Qty["Shared Storage"].Subtotal;
	accountItemQty[name].Total=accountItemQty[name].Total+total-oldTotal;
	accountItemQty[name].Qty["Shared Storage"]={["Subtotal"]=total};
	for k,v in pairs(containerQty) do
		accountItemQty[name].Qty["Shared Storage"][k]=v;
	end
	-- now update the display
	if inventoryWindow.CharList:GetSelectedIndex()==2 then
		if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
		if accountItemQty[name].ListItem["Shared Storage"]==nil then
			-- create a new entry, set height by filter and position it according to the sort
			local tmpEntry=ItemEntry(name,"Shared Storage",accountItemQty[name].Qty["Shared Storage"].Subtotal,itemInfo)
			accountItemQty[name].ListItem["Shared Storage"]=tmpEntry;
			inventoryPanel.ItemList:AddItem(tmpEntry)
			inventoryPanel.SortList:SelectedIndexChanged();
		else
			accountItemQty[name].ListItem["Shared Storage"].Qty:SetText(accountItemQty[name].Qty["Shared Storage"].Subtotal);
		end
	elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
		if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
		if accountItemQty[name].ListItem["ALL"]==nil then
			-- create a new entry, set height by filter and position it according to the sort
			local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
			accountItemQty[name].ListItem["ALL"]=tmpEntry;
			inventoryPanel.ItemList:AddItem(tmpEntry)
			inventoryPanel.SortList:SelectedIndexChanged();
		else
			accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
		end
	end
end
sharedStorageImage={};
function refreshSharedStorage()
	if Settings.debug then
		Turbine.Shell.WriteLine("refreshSharedStorage")
	end
	if sharedStorage:IsAvailable() then
		local tmpItemFlag={}
		-- clear out any accountItemQty record for "Shared Storage" and recalc the total for any item that had a "Shared Storage" qty

		for k,v in pairs(sharedStorageImage) do
			if v.Item~=nil then
				-- safely remove old callbacks
				if v.Item.QuantityChanged~=nil then
					RemoveCallback(v.Item,"QuantityChanged",sharedStorageQuantityChanged);
					RemoveCallback(v.Item,"ChestChanged",sharedStorageChestChanged);
				end
			end
		end
		sharedStorageImage={};
		-- set flags for items that were in the old vault
		for k,v in pairs(accountItemQty) do
			local total=0;
			for name,qty in pairs(v.Qty) do
				if name=="Shared Storage" then
					if type(qty)~="table" then
						tmpItemFlag[k]=qty
						accountItemQty[k].Qty["Shared Storage"]={["Subtotal"]=qty};
					else
						tmpItemFlag[k]=qty.Subtotal;
						accountItemQty[k].Qty["Shared Storage"]={["Subtotal"]=qty.Subtotal};
					end
				end
			end
		end
		-- process the current Shared Storage, adding an accountItemQty record and updating total
		local size=sharedStorage:GetCount();
		CharList["Shared Storage"].used=size;
		CharList["Shared Storage"].capacity=sharedStorage:GetCapacity();
		if inventoryWindow.CharList:GetSelectedIndex()==2 then
			inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList["Shared Storage"].used).."/"..tostring(CharList["Shared Storage"].capacity)..")")
		end

		local tmpIndex,tmpItem;
		for tmpIndex=1,size do
			tmpItem=sharedStorage:GetItem(tmpIndex);
			sharedStorageImage[tmpIndex]={}
			sharedStorageImage[tmpIndex].Item=tmpItem;
			if tmpItem~=nil then
				local itemInfo=tmpItem:GetItemInfo();
				local name=itemInfo:GetName();
				local qty=tmpItem:GetQuantity();
				sharedStorageImage[tmpIndex].Index=tmpIndex;
				AddCallback(sharedStorageImage[tmpIndex].Item,"QuantityChanged",sharedStorageQuantityChanged);
				AddCallback(sharedStorageImage[tmpIndex].Item,"ChestChanged",sharedStorageChestChanged);
				sharedStorageImage[tmpIndex].Item.Chest=sharedStorageImage[tmpIndex].Item:GetChest();
				sharedStorageImage[tmpIndex].ItemName=name;
				if accountItemQty[name]==nil then
					accountItemQty[name]={}
					accountItemQty[name].Total=0;
					accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
					accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
					accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
					accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
					accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
					accountItemQty[name].Qty={};
					accountItemQty[name].ListItem={};
					local found=false;
					for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
						if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
							if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetSelectedIndex()==2 then
								inventoryPanel.FilterList:ShowEntry(filterIndex);
							end
							found=true;
							break;
						end
					end
					if not found then
						accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
						if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetSelectedIndex()==2 then
							inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
						end
					end
				end
				if accountItemQty[name].ItemInfo==nil then
					accountItemQty[name].ItemInfo=itemInfo;
				end
				if inventoryWindow.CharList:GetSelectedIndex()==2 and accountItemQty[name].ListItem["Shared Storage"]==nil then
					local tmpEntry=ItemEntry(name,"Shared Storage",qty,itemInfo);					
					inventoryPanel.ItemList:AddItem(tmpEntry)
					accountItemQty[name].ListItem["Shared Storage"]=tmpEntry;
				end
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[name].ListItem["ALL"]==nil then
					local tmpEntry=ItemEntry(name,"ALL",qty,itemInfo);					
					inventoryPanel.ItemList:AddItem(tmpEntry)
					accountItemQty[name].ListItem["ALL"]=tmpEntry;
				end
				if accountItemQty[name].Qty["Shared Storage"]==nil then
					accountItemQty[name].Qty["Shared Storage"]={["Subtotal"]=qty};
				else
					if tmpItemFlag[name]==nil then
						accountItemQty[name].Qty["Shared Storage"].Subtotal=accountItemQty[name].Qty["Shared Storage"].Subtotal+qty;
					else
						accountItemQty[name].Total=accountItemQty[name].Total-tmpItemFlag[name];
						tmpItemFlag[name]=nil;
						accountItemQty[name].Qty["Shared Storage"].Subtotal=qty;
					end
				end
				local bag=tmpItem:GetChest();
				if accountItemQty[name].Qty["Shared Storage"][bag]==nil then
					accountItemQty[name].Qty["Shared Storage"][bag]=qty;
				else
					accountItemQty[name].Qty["Shared Storage"][bag]=accountItemQty[name].Qty["Shared Storage"][bag]+qty;
				end
				if inventoryWindow.CharList:GetSelectedIndex()==2 and accountItemQty[name].ListItem["Shared Storage"].ItemInfo~=nil then
					accountItemQty[name].ListItem["Shared Storage"].ItemInfo:SetItemInfo(itemInfo);
					accountItemQty[name].ListItem["Shared Storage"].IconBorder:SetVisible(false);
					accountItemQty[name].ListItem["Shared Storage"].Qty:SetText(accountItemQty[name].Qty["Shared Storage"].Subtotal);
				end
				accountItemQty[name].Total=accountItemQty[name].Total+qty;
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[name].ListItem["ALL"]~=nil and accountItemQty[name].ListItem["ALL"].ItemInfo~=nil then
					accountItemQty[name].ListItem["ALL"].ItemInfo:SetItemInfo(itemInfo);
					accountItemQty[name].ListItem["ALL"].IconBorder:SetVisible(false);
					accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
				end
			end
		end
		for tmpItem,val in pairs(tmpItemFlag) do
			if inventoryWindow.CharList:GetSelectedIndex()==2 and accountItemQty[tmpItem].ListItem["Shared Storage"]~=nil then
				-- remove the linked display entry
				local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItemQty[tmpItem].ListItem["Shared Storage"])
				if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
				accountItemQty[tmpItem].ListItem["Shared Storage"]=nil;
			end

			-- these items existed previously and now have to be accounted for
			-- recalc the total instead of adjusting just in case the discrepancy was due to a character rename, data corruption or playing the account on another computer
			accountItemQty[tmpItem].Qty["Shared Storage"]=nil;
			local total=0;
			for char,qty in pairs(accountItemQty[tmpItem].Qty) do
				total=total+qty.Subtotal;
			end
			if total==0 then
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[tmpItem].ListItem~=nil and accountItemQty[tmpItem].ListItem["ALL"]~=nil then
					-- remove the "ALL" linked display entry
					local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItemQty[tmpItem].ListItem["ALL"])
					if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
					accountItemQty[tmpItem].ListItem["ALL"]=nil;
				end
				accountItemQty[tmpItem]=nil;
			else
				accountItemQty[tmpItem].Total=total;
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[tmpItem].ListItem~=nil and accountItemQty[tmpItem].ListItem["ALL"]~=nil then
					accountItemQty[tmpItem].ListItem["ALL"].Qty:SetText(total);
				end
			end
		end
		-- now reapply the sort
		if inventoryWindow.CharList:GetSelectedIndex()==2 or inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryPanel.SortList:SelectedIndexChanged();
		end
	end
end
AddCallback(sharedStorage,"IsAvailableChanged",refreshSharedStorage);
AddCallback(sharedStorage,"ItemsRefreshed",refreshSharedStorage);
sharedStorageItemAdded=function(sender,args)
	-- Index, Item
	if Settings.debug then
		Turbine.Shell.WriteLine("sharedStorageItemAdded");
	end
	local index=args.Index;
	local item=args.Item;
	table.insert(sharedStorageImage,index,{});

	CharList["Shared Storage"].used=sharedStorage:GetCount();
	if inventoryWindow.CharList:GetSelectedIndex()==2 then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList["Shared Storage"].used).."/"..tostring(CharList["Shared Storage"].capacity)..")")
	end
	sharedStorageImage[index].Item=item;
	if item~=nil then
		local itemInfo=item:GetItemInfo();
		local name=itemInfo:GetName();
		local qty=item:GetQuantity();
		sharedStorageImage[index].Index=tmpIndex;
		AddCallback(sharedStorageImage[index].Item,"QuantityChanged",sharedStorageQuantityChanged);
		AddCallback(sharedStorageImage[index].Item,"ChestChanged",sharedStorageChestChanged);
		sharedStorageImage[index].Item.Chest=sharedStorageImage[index].Item:GetChest();
		sharedStorageImage[index].ItemName=name;
		if accountItemQty[name]==nil then
			accountItemQty[name]={}
			accountItemQty[name].Total=0;
			accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
			accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
			accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
			accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].Qty={};
			accountItemQty[name].ListItem={};
			local found=false;
			for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
				if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
					if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetSelectedIndex()==2 then
						inventoryPanel.FilterList:ShowEntry(filterIndex);
					end
					found=true;
					break;
				end
			end
			if not found then
				accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
				if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetSelectedIndex()==2 then
					inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
				end
			end
		end
		if accountItemQty[name].ItemInfo==nil then
			accountItemQty[name].ItemInfo=itemInfo;
			if accountItemQty[name].ListItem["Shared Storage"]~=nil then
				accountItemQty[name].ListItem["Shared Storage"].ItemInfo:SetItemInfo(itemInfo);
				accountItemQty[name].ListItem["Shared Storage"].IconBorder:SetVisible(false);
			end
		end
		if accountItemQty[name].Qty["Shared Storage"]==nil then
			accountItemQty[name].Qty["Shared Storage"]={["Subtotal"]=qty};
		else
			accountItemQty[name].Qty["Shared Storage"].Subtotal=accountItemQty[name].Qty["Shared Storage"].Subtotal+qty;
		end
		local bag=item:GetChest();
		if accountItemQty[name].Qty["Shared Storage"][bag]==nil then
			accountItemQty[name].Qty["Shared Storage"][bag]=qty;
		else
			accountItemQty[name].Qty["Shared Storage"][bag]=accountItemQty[name].Qty["Shared Storage"][bag]+qty;
		end
		accountItemQty[name].Total=accountItemQty[name].Total+qty;
		-- now update the display element
		if inventoryWindow.CharList:GetSelectedIndex()==2 then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if accountItemQty[name].ListItem["Shared Storage"]==nil then
				-- create a new entry, set height by filter and position it according to the sort
				local tmpEntry=ItemEntry(name,"Shared Storage",accountItemQty[name].Qty["Shared Storage"].Subtotal,itemInfo)
				accountItemQty[name].ListItem["Shared Storage"]=tmpEntry;
				inventoryPanel.ItemList:AddItem(tmpEntry)
				if tmpEntry:GetHeight()>0 then
					inventoryPanel.SortList:SelectedIndexChanged();
				end
			else
				accountItemQty[name].ListItem["Shared Storage"].Qty:SetText(accountItemQty[name].Qty["Shared Storage"].Subtotal);
			end
		elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if accountItemQty[name].ListItem["ALL"]==nil then
				-- create a new entry, set height by filter and position it according to the sort
				local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
				accountItemQty[name].ListItem["ALL"]=tmpEntry;
				inventoryPanel.ItemList:AddItem(tmpEntry)
				inventoryPanel.SortList:SelectedIndexChanged();
			else
				accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
			end
		end
		-- now reapply the sort
		if inventoryWindow.CharList:GetSelectedIndex()==2 or inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryPanel.SortList:SelectedIndexChanged();
		end
	end
end
sharedStorageItemRemoved=function(sender,args)
-- Index, Item
	if Settings.debug then
		Turbine.Shell.WriteLine("sharedStorageItemRemoved");
	end
	local index=args.Index;
	local name=args.Item:GetItemInfo():GetName();
	local item=args.Item;
	local qty=0;
	if item~=nil then
		qty=item:GetQuantity();
		if item.QuantityChanged~=nil then
			RemoveCallback(item,"QuantityChanged",sharedStorageQuantityChanged);
			RemoveCallback(item,"ChestChanged",sharedStorageChestChanged);
		end
	end
	table.remove(sharedStorageImage,index);
	local accountItem=accountItemQty[name];
	if accountItem~=nil then
		accountItem.Total=accountItem.Total-qty;
		if accountItem.Qty["Shared Storage"]~=nil then
			local totQty=accountItem.Qty["Shared Storage"].Subtotal-qty
			if totQty==0 then
				accountItem.Qty["Shared Storage"]=nil
				-- remove the display entry
				if inventoryWindow.CharList:GetSelectedIndex()==2 and accountItem.ListItem["Shared Storage"]~=nil then
					-- remove the linked display entry
					local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem["Shared Storage"])
					if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
					accountItem.ListItem["Shared Storage"]=nil;
				end
			else
				local bag=item:GetChest();
				accountItem.Qty["Shared Storage"][bag]=accountItem.Qty["Shared Storage"][bag]-qty;
				if accountItem.Qty["Shared Storage"][bag]==0 then accountItem.Qty["Shared Storage"][bag]=nil end
				accountItem.Qty["Shared Storage"].Subtotal=totQty
				if inventoryWindow.CharList:GetSelectedIndex()==2 and accountItem.ListItem["Shared Storage"]~=nil then
					-- update the linked display entry
					accountItem.ListItem["Shared Storage"].Qty:SetText(totQty);
				end
			end
		end
		if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItem.ListItem["ALL"]~=nil then
			if accountItem.Total==0 then
				-- remove the linked display entry
				local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem["ALL"])
				if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
				accountItem.ListItem["ALL"]=nil;
			else
				accountItem.ListItem["ALL"].Qty:SetText(accountItem.Total);
			end
		end
	end
	CharList["Shared Storage"].used=CharList["Shared Storage"].used-1; -- can't use the GetCount since this event is fired BEFORE the item is actually removed
	if inventoryWindow.CharList:GetSelectedIndex()==2 then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList["Shared Storage"].used).."/"..tostring(CharList["Shared Storage"].capacity)..")")
	end
end
AddCallback(sharedStorage,"ItemAdded",sharedStorageItemAdded);

AddCallback(sharedStorage,"ItemRemoved",sharedStorageItemRemoved);

-- Vault
vault=localPlayer:GetVault();
function vaultChestChanged(sender)
	-- sender.Chest== old chest
	local oldChest=sender.Chest;
	local newChest=sender:GetChest();
	local qty=sender:GetQuantity();
	local name=sender:GetItemInfo():GetName();
	accountItemQty[name].Qty[charVaultName][oldChest]=accountItemQty[name].Qty[charVaultName][oldChest]-qty;
	if accountItemQty[name].Qty[charVaultName][oldChest]==0 then accountItemQty[name].Qty[charVaultName][oldChest]=nil end
	if accountItemQty[name].Qty[charVaultName][newChest]==nil then
		accountItemQty[name].Qty[charVaultName][newChest]=qty;
	else
		accountItemQty[name].Qty[charVaultName][newChest]=accountItemQty[name].Qty[charVaultName][newChest]+qty;
	end
	sender.Chest=newChest;
end
function vaultQuantityChanged(sender,args)
	if Settings.debug then
		Turbine.Shell.WriteLine("vaultQuantityChanged")
		Turbine.Shell.WriteLine("sender:"..tostring(sender))
	end
	local name=sender:GetItemInfo():GetName();
	local index;

	-- scan the vault, recalc the total for this name and then update accountItemQty
	local total=0;
	local containerQty={};
	for index=1,vault:GetCount() do
		local item=vault:GetItem(index);
		if item~=nil and item:GetItemInfo():GetName()==name then
			local tmpQty=item:GetQuantity();
			local bag=item:GetChest();
			total=total+tmpQty;
			if containerQty[bag]==nil then
				containerQty[bag]=tmpQty;
			else
				containerQty[bag]=containerQty[bag]+tmpQty;
			end
		end
	end
	local oldTotal=accountItemQty[name].Qty[charVaultName].Subtotal;
	accountItemQty[name].Total=accountItemQty[name].Total+total-oldTotal;
	accountItemQty[name].Qty[charVaultName]={["Subtotal"]=total};
	for k,v in pairs(containerQty) do
		accountItemQty[name].Qty[charVaultName][k]=v;
	end
	-- now update the display
	if inventoryWindow.CharList:GetText()==charVaultName then
		if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
		if accountItemQty[name].ListItem[charVaultName]==nil then
			-- create a new entry, set height by filter and position it according to the sort
			local tmpEntry=ItemEntry(name,charVaultName,accountItemQty[name].Qty[charVaultName].Subtotal,itemInfo)
			accountItemQty[name].ListItem[charVaultName]=tmpEntry;
			inventoryPanel.ItemList:AddItem(tmpEntry)
			inventoryPanel.SortList:SelectedIndexChanged();
		else
			accountItemQty[name].ListItem[charVaultName].Qty:SetText(accountItemQty[name].Qty[charVaultName].Subtotal);
		end
	elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
		if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
		if accountItemQty[name].ListItem["ALL"]==nil then
			-- create a new entry, set height by filter and position it according to the sort
			local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
			accountItemQty[name].ListItem["ALL"]=tmpEntry;
			inventoryPanel.ItemList:AddItem(tmpEntry)
			inventoryPanel.SortList:SelectedIndexChanged();
		else
			accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
		end
	end
end
vaultImage={};
function refreshVault()
	if Settings.debug then
		Turbine.Shell.WriteLine("refreshVault")
	end
	if vault:IsAvailable() then
		local tmpItemFlag={}
		-- clear out any accountItemQty record for charVaultName and recalc the total for any item that had a charVaultName qty

		for k,v in pairs(vaultImage) do
			if v.Item~=nil then
				-- safely remove old callbacks
				if v.Item.QuantityChanged~=nil then
					RemoveCallback(v.Item,"QuantityChanged",vaultQuantityChanged);
					RemoveCallback(v.Item,"ChestChanged",vaultChestChanged);
				end
			end
		end
		vaultImage={};
		-- set flags for items that were in the old vault
		for k,v in pairs(accountItemQty) do
			local total=0;
			for name,qty in pairs(v.Qty) do
				if name==charVaultName then
					tmpItemFlag[k]=qty.Subtotal;
					accountItemQty[k].Qty[charVaultName]={["Subtotal"]=qty.Subtotal};
				end
			end
		end
		-- process the current Shared Storage, adding an accountItemQty record and updating total
		local size=vault:GetCount();
		CharList[charVaultName].used=size;
		CharList[charVaultName].capacity=vault:GetCapacity();
		if inventoryWindow.CharList:GetText()==charVaultName then
			inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charVaultName].used).."/"..tostring(CharList[charVaultName].capacity)..")")
		end

		local tmpIndex,tmpItem;
		for tmpIndex=1,size do
			tmpItem=vault:GetItem(tmpIndex);
			vaultImage[tmpIndex]={}
			vaultImage[tmpIndex].Item=tmpItem;
			if tmpItem~=nil then
				local itemInfo=tmpItem:GetItemInfo();
				local name=itemInfo:GetName();
				local qty=tmpItem:GetQuantity();
				vaultImage[tmpIndex].Index=tmpIndex;
				AddCallback(vaultImage[tmpIndex].Item,"QuantityChanged",vaultQuantityChanged);
				AddCallback(vaultImage[tmpIndex].Item,"ChestChanged",vaultChestChanged);
				vaultImage[tmpIndex].Item.Chest=vaultImage[tmpIndex].Item:GetChest();
				vaultImage[tmpIndex].ItemName=name;
				if accountItemQty[name]==nil then
					accountItemQty[name]={}
					accountItemQty[name].Total=0;
					accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
					accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
					accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
					accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
					accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
					accountItemQty[name].Qty={};
					accountItemQty[name].ListItem={};
					local found=false;
					for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
						if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
							if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charVaultName then
								inventoryPanel.FilterList:ShowEntry(filterIndex);
							end
							found=true;
							break;
						end
					end
					if not found then
						accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
						if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charVaultName then
							inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
						end
					end
				end
				if accountItemQty[name].ItemInfo==nil then
					accountItemQty[name].ItemInfo=itemInfo;
				end
				if inventoryWindow.CharList:GetText()==charVaultName and accountItemQty[name].ListItem[charVaultName]==nil then
					local tmpEntry=ItemEntry(name,charVaultName,qty,itemInfo);					
					inventoryPanel.ItemList:AddItem(tmpEntry)
					accountItemQty[name].ListItem[charVaultName]=tmpEntry;
				end
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[name].ListItem["ALL"]==nil then
					local tmpEntry=ItemEntry(name,"ALL",qty,itemInfo);					
					inventoryPanel.ItemList:AddItem(tmpEntry)
					accountItemQty[name].ListItem["ALL"]=tmpEntry;
				end
				if accountItemQty[name].Qty[charVaultName]==nil then
					accountItemQty[name].Qty[charVaultName]={["Subtotal"]=qty};
				else
					if tmpItemFlag[name]==nil then
						accountItemQty[name].Qty[charVaultName].Subtotal=accountItemQty[name].Qty[charVaultName].Subtotal+qty;
					else
						accountItemQty[name].Total=accountItemQty[name].Total-tmpItemFlag[name];
						tmpItemFlag[name]=nil;
						accountItemQty[name].Qty[charVaultName].Subtotal=qty;
					end
				end
				local bag=tmpItem:GetChest();
				if accountItemQty[name].Qty[charVaultName][bag]==nil then
					accountItemQty[name].Qty[charVaultName][bag]=qty;
				else
					accountItemQty[name].Qty[charVaultName][bag]=accountItemQty[name].Qty[charVaultName][bag]+qty;
				end
				if inventoryWindow.CharList:GetText()==charVaultName and accountItemQty[name].ListItem[charVaultName].ItemInfo~=nil then
					accountItemQty[name].ListItem[charVaultName].ItemInfo:SetItemInfo(itemInfo);
					accountItemQty[name].ListItem[charVaultName].IconBorder:SetVisible(false);
					accountItemQty[name].ListItem[charVaultName].Qty:SetText(accountItemQty[name].Qty[charVaultName].Subtotal);
				end
				accountItemQty[name].Total=accountItemQty[name].Total+qty;
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[name].ListItem["ALL"]~=nil and accountItemQty[name].ListItem["ALL"].ItemInfo~=nil then
					accountItemQty[name].ListItem["ALL"].ItemInfo:SetItemInfo(itemInfo);
					accountItemQty[name].ListItem["ALL"].IconBorder:SetVisible(false);
					accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
				end
			end
		end
		for tmpItem,val in pairs(tmpItemFlag) do
			if inventoryWindow.CharList:GetText()==charVaultName and accountItemQty[tmpItem].ListItem[charVaultName]~=nil then
				-- remove the linked display entry
				local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItemQty[tmpItem].ListItem[charVaultName])
				if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
				accountItemQty[tmpItem].ListItem[charVaultName]=nil;
			end

			-- these items existed previously and now have to be accounted for
			-- recalc the total instead of adjusting just in case the discrepancy was due to a character rename, data corruption or playing the account on another computer
			accountItemQty[tmpItem].Qty[charVaultName]=nil;
			local total=0;
			for char,qty in pairs(accountItemQty[tmpItem].Qty) do
				total=total+qty.Subtotal;
			end
			if total==0 then
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[tmpItem].ListItem~=nil and accountItemQty[tmpItem].ListItem["ALL"]~=nil then
					-- remove the "ALL" linked display entry
					local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItemQty[tmpItem].ListItem["ALL"])
					if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
					accountItemQty[tmpItem].ListItem["ALL"]=nil;
				end
				accountItemQty[tmpItem]=nil;
			else
				accountItemQty[tmpItem].Total=total;
				if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItemQty[tmpItem].ListItem~=nil and accountItemQty[tmpItem].ListItem["ALL"]~=nil then
					accountItemQty[tmpItem].ListItem["ALL"].Qty:SetText(total);
				end
			end
		end
		-- now reapply the sort
		if inventoryWindow.CharList:GetText()==charVaultName or inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryPanel.SortList:SelectedIndexChanged();
		end
	end
end
AddCallback(vault,"IsAvailableChanged",refreshVault);
AddCallback(vault,"ItemsRefreshed",refreshVault);
vaultItemAdded=function(sender,args)
	-- Index, Item
	if Settings.debug then
		Turbine.Shell.WriteLine("vaultItemAdded");
	end
	local index=args.Index;
	local item=args.Item;
	table.insert(vaultImage,index,{});

	CharList[charVaultName].used=vault:GetCount();
	if inventoryWindow.CharList:GetText()==charVaultName then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charVaultName].used).."/"..tostring(CharList[charVaultName].capacity)..")")
	end
	vaultImage[index].Item=item;
	if item~=nil then
		local itemInfo=item:GetItemInfo()
		local name=itemInfo:GetName();
		local qty=item:GetQuantity();
		vaultImage[index].Index=tmpIndex;
		AddCallback(vaultImage[index].Item,"QuantityChanged",vaultQuantityChanged);
		AddCallback(vaultImage[index].Item,"ChestChanged",vaultChestChanged);
		vaultImage[index].Item.Chest=vaultImage[index].Item:GetChest();
		vaultImage[index].ItemName=name;
		if accountItemQty[name]==nil then
			accountItemQty[name]={}
			accountItemQty[name].Total=0;
			accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
			accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
			accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
			accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].Qty={};
			accountItemQty[name].ListItem={};
			local found=false;
			for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
				if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
					if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charVaultName then
						inventoryPanel.FilterList:ShowEntry(filterIndex);
					end
					found=true;
					break;
				end
			end
			if not found then
				accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
				if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charVaultName then
					inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
				end
			end
		end
		if accountItemQty[name].ItemInfo==nil then
			accountItemQty[name].ItemInfo=itemInfo;
			if accountItemQty[name].ListItem[charVaultName]~=nil then
				accountItemQty[name].ListItem[charVaultName].ItemInfo:SetItemInfo(itemInfo);
				accountItemQty[name].ListItem[charVaultName].IconBorder:SetVisible(false);
			end
		end
		if accountItemQty[name].Qty[charVaultName]==nil then
			accountItemQty[name].Qty[charVaultName]={["Subtotal"]=qty};
		else
			accountItemQty[name].Qty[charVaultName].Subtotal=accountItemQty[name].Qty[charVaultName].Subtotal+qty;
		end
		local bag=item:GetChest();
		if accountItemQty[name].Qty[charVaultName][bag]==nil then
			accountItemQty[name].Qty[charVaultName][bag]=qty;
		else
			accountItemQty[name].Qty[charVaultName][bag]=accountItemQty[name].Qty[charVaultName][bag]+qty;
		end
		accountItemQty[name].Total=accountItemQty[name].Total+qty;
		-- now update the display element
		if inventoryWindow.CharList:GetText()==charVaultName then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if accountItemQty[name].ListItem[charVaultName]==nil then
				-- create a new entry, set height by filter and position it according to the sort
				local tmpEntry=ItemEntry(name,charVaultName,accountItemQty[name].Qty[charVaultName].Subtotal,itemInfo)
				accountItemQty[name].ListItem[charVaultName]=tmpEntry;
				inventoryPanel.ItemList:AddItem(tmpEntry)
				if tmpEntry:GetHeight()>0 then
					inventoryPanel.SortList:SelectedIndexChanged();
				end
			else
				accountItemQty[name].ListItem[charVaultName].Qty:SetText(accountItemQty[name].Qty[charVaultName].Subtotal);
			end
		elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if accountItemQty[name].ListItem["ALL"]==nil then
				-- create a new entry, set height by filter and position it according to the sort
				local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
				accountItemQty[name].ListItem["ALL"]=tmpEntry;
				inventoryPanel.ItemList:AddItem(tmpEntry)
				inventoryPanel.SortList:SelectedIndexChanged();
			else
				accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
			end
		end
		-- now reapply the sort
		if inventoryWindow.CharList:GetText()==charVaultName or inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryPanel.SortList:SelectedIndexChanged();
		end
	end
end
vaultItemRemoved=function(sender,args)
-- Index, Item
	if Settings.debug then
		Turbine.Shell.WriteLine("vaultItemRemoved");
	end
	local index=args.Index;
	local name=args.Item:GetItemInfo():GetName();
	local item=args.Item;
	local qty=0;
	if item~=nil then
		qty=item:GetQuantity();
		if item.QuantityChanged~=nil then
			RemoveCallback(item,"QuantityChanged",vaultQuantityChanged);
			RemoveCallback(item,"ChestChanged",vaultChestChanged);
		end
	end
	table.remove(vaultImage,index);
	local accountItem=accountItemQty[name];
 	if accountItem~=nil then
		accountItem.Total=accountItem.Total-qty;
		if accountItem.Qty[charVaultName]~=nil then
			local totQty=accountItem.Qty[charVaultName].Subtotal-qty
			if totQty==0 then
				accountItem.Qty[charVaultName]=nil
				-- remove the display entry
				if inventoryWindow.CharList:GetText()==charVaultName and accountItem.ListItem[charVaultName]~=nil then
					-- remove the linked display entry
					local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem[charVaultName])
					if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
					accountItem.ListItem[charVaultName]=nil;
				end
			else
				local bag=item:GetChest();
				accountItem.Qty[charVaultName][bag]=accountItem.Qty[charVaultName][bag]-qty;
				if accountItem.Qty[charVaultName][bag]==0 then accountItem.Qty[charVaultName][bag]=nil end
				accountItem.Qty[charVaultName].Subtotal=totQty;
				if inventoryWindow.CharList:GetText()==charVaultName and accountItem.ListItem[charVaultName]~=nil then
					-- update the linked display entry
					accountItem.ListItem[charVaultName].Qty:SetText(totQty);
				end
			end
		end
		if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItem.ListItem["ALL"]~=nil then
			if accountItem.Total==0 then
				-- remove the linked display entry
				local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem["ALL"])
				if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
				accountItem.ListItem["ALL"]=nil;
			else
				accountItem.ListItem["ALL"].Qty:SetText(accountItem.Total);
			end
		end
	end
	CharList[charVaultName].used=CharList[charVaultName].used-1; -- can't use the GetCount since this event is fired BEFORE the item is actually removed
	if inventoryWindow.CharList:GetText()==charVaultName then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charVaultName].used).."/"..tostring(CharList[charVaultName].capacity)..")")
	end
end
AddCallback(vault,"ItemAdded",vaultItemAdded);
AddCallback(vault,"ItemRemoved",vaultItemRemoved);

function refreshBackPackImage()
	if Settings.debug then
		Turbine.Shell.WriteLine("refreshBackPackImage")
	end
	local used=0;
	local index
	for index=1,backPack:GetSize() do
		if backpackImage[index]~=nil and backpackImage[index].Item~=nil then
			-- safely remove old callbacks
			if backpackImage[index].Item.QuantityChanged~=nil then
				RemoveCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged)
			end
		end
		backpackImage[index]={};
		backpackImage[index].Item=backPack:GetItem(index);
		if backpackImage[index].Item~=nil then
			used=used+1;
			backpackImage[index].Index=index;
			AddCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged);
			backpackImage[index].Item.Index=index;
			backpackImage[index].ItemName=backpackImage[index].Item:GetItemInfo():GetName();
		end
	end
	CharList[charName].used=used;
	CharList[charName].capacity=backPack:GetSize()
	if inventoryWindow.CharList:GetText()==charName then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charName].used).."/"..tostring(CharList[charName].capacity)..")")
	end
end
refreshBackPackImage()
itemAdded=function(sender,args)
	if Settings.debug then
		Turbine.Shell.WriteLine("itemAdded")
		Table.Dump(args)
		Turbine.Shell.WriteLine("name:"..backPack:GetItem(args.Index):GetName())
	end
	-- Index, Item
	local index=args.Index;
	if backpackImage[index]~=nil and backpackImage[index].Item~=nil then
		-- safely remove old callbacks
		if backpackImage[index].Item.QuantityChanged~=nil then
			RemoveCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged)
		end
		backpackImage[index].ItemName=nil;
		backpackImage[index].Item=nil;
		backpackImage[index].Index=nil;
		backpackImage[index]=nil;
	end
	backpackImage[index]={};
	backpackImage[index].Item=backPack:GetItem(index);
	if backpackImage[index].Item~=nil then
		CharList[charName].used=CharList[charName].used+1;
		if inventoryWindow.CharList:GetText()==charName then
			inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charName].used).."/"..tostring(CharList[charName].capacity)..")")
		end
		backpackImage[index].Index=index;
		AddCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged)
		backpackImage[index].Item.Index=index;
		-- we need to track the item name for itemRemoved events
		local itemInfo=backpackImage[index].Item:GetItemInfo();
		local name=itemInfo:GetName();
		local qty=backpackImage[index].Item:GetQuantity();
		if accountItemQty[name]==nil then
			accountItemQty[name]={}
			accountItemQty[name].Total=0;
			accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
			accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
			accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
			accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
			accountItemQty[name].Qty={};
			accountItemQty[name].ListItem={};
			local found=false;
			for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
				if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
					if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charName then
						inventoryPanel.FilterList:ShowEntry(filterIndex);
					end
					found=true;
					break;
				end
			end
			if not found then
				accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
				if inventoryWindow.CharList:GetSelectedIndex()==1 or inventoryWindow.CharList:GetText()==charName then
					inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
				end
			end
		end
		if accountItemQty[name].ItemInfo==nil then
			accountItemQty[name].ItemInfo=itemInfo;
			if inventoryWindow.CharList:GetText()==charName and accountItemQty[name].ListItem[charName]~=nil then
				if Settings.totalsOnly then
					accountItemQty[name].ListItem[charName].ItemInfo:SetItemInfo(itemInfo);
					accountItemQty[name].ListItem[charName].IconBorder:SetVisible(false);
				else
					if accountItemQty[name].ListItem[charName][index]~=nil then
						accountItemQty[name].ListItem[charName][index].ItemInfo:SetItemInfo(itemInfo);
						accountItemQty[name].ListItem[charName][index].IconBorder:SetVisible(false);
					end
				end
			end
		end
		if accountItemQty[name].Qty[charName]==nil then
			accountItemQty[name].Qty[charName]={["Subtotal"]=qty};
		else
			accountItemQty[name].Qty[charName].Subtotal=accountItemQty[name].Qty[charName].Subtotal+qty;
		end
		accountItemQty[name].Qty[charName][index]=qty
		accountItemQty[name].Total=accountItemQty[name].Total+qty;
		backpackImage[index].ItemName=name
		-- update display
		if inventoryWindow.CharList:GetText()==charName then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if Settings.totalsOnly then
				if accountItemQty[name].ListItem[charName]==nil then
					-- create a new entry, set height by filter and position it according to the sort
					local tmpEntry=ItemEntry(name,charName,accountItemQty[name].Qty[charName].Subtotal,itemInfo)
					if not Settings.totalsOnly then
						tmpEntry.Item:SetItem(backPack:GetItem(index))
						tmpEntry.Item.Index=index;
					end
					accountItemQty[name].ListItem[charName]=tmpEntry;
					inventoryPanel.ItemList:AddItem(tmpEntry)
					if tmpEntry:GetHeight()>0 then
						inventoryPanel.SortList:SelectedIndexChanged();
					end
				else
					accountItemQty[name].ListItem[charName].Qty:SetText(accountItemQty[name].Qty[charName].Subtotal);
				end
			else
				if accountItemQty[name].ListItem[charName]==nil then accountItemQty[name].ListItem[charName]={} end
				if accountItemQty[name].ListItem[charName][index]==nil then
					-- create a new entry, set height by filter and position it according to the sort
					local tmpEntry=ItemEntry(name,charName,qty,itemInfo)
					if not Settings.totalsOnly then
						tmpEntry.Item:SetItem(backPack:GetItem(index))
						tmpEntry.Item.Index=index;
					end
					accountItemQty[name].ListItem[charName][index]=tmpEntry;
					inventoryPanel.ItemList:AddItem(tmpEntry)
					if tmpEntry:GetHeight()>0 then
						inventoryPanel.SortList:SelectedIndexChanged();
					end
				else
					accountItemQty[name].ListItem[charName][index].Qty:SetText(qty);
				end
			end
		elseif inventoryWindow.CharList:GetSelectedIndex()==1 then
			if accountItemQty[name].ListItem==nil then accountItemQty[name].ListItem={} end
			if accountItemQty[name].ListItem["ALL"]==nil then
				-- create a new entry, set height by filter and position it according to the sort
				local tmpEntry=ItemEntry(name,"ALL",accountItemQty[name].Total,itemInfo)
				accountItemQty[name].ListItem["ALL"]=tmpEntry;
				inventoryPanel.ItemList:AddItem(tmpEntry)
				inventoryPanel.SortList:SelectedIndexChanged();
			else
				accountItemQty[name].ListItem["ALL"].Qty:SetText(accountItemQty[name].Total);
			end
		end
		-- now reapply the sort
		if inventoryWindow.CharList:GetText()==charName or inventoryWindow.CharList:GetSelectedIndex()==1 then
			inventoryPanel.SortList:SelectedIndexChanged();
		end
	end
end
itemRemoved=function(sender,args)
	if Settings.debug then
		Turbine.Shell.WriteLine("itemRemoved")
		Table.Dump(args)
		Turbine.Shell.WriteLine("name:"..backPack:GetItem(args.Index):GetName())
	end
	-- Index
	local index=args.Index;
	local bag=math.floor((index+14)/15);
	if backpackImage[index]~=nil and backpackImage[index].Item~=nil then
		-- safely remove old callbacks
		if backpackImage[index].Item.QuantityChanged~=nil then
			RemoveCallback(backpackImage[index].Item,"QuantityChanged",quantityChanged);
		end
		local name=backpackImage[index].ItemName;
		local item=backpackImage[index].Item;
		local qty=item:GetQuantity();
		local accountItem=accountItemQty[name];
		if accountItem~=nil then
			accountItem.Total=accountItem.Total-qty;
			if accountItem.Qty[charName]~=nil then
				totQty=accountItem.Qty[charName].Subtotal-qty;
				accountItem.Qty[charName][index]=nil;
				if inventoryWindow.CharList:GetText()==charName and accountItem.ListItem[charName]~=nil then
					-- remove the linked display entry
					if Settings.totalsOnly then
						if totQty==0 then
							local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem[charName])
							if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
							accountItem.ListItem[charName]=nil;
						else
							accountItem.ListItem[charName].Qty:SetText(totQty);
						end
					else
						if accountItem.ListItem[charName][index]~=nil then
							local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem[charName][index])
							if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
							accountItem.ListItem[charName][index]=nil;
						end
					end
				end
				if totQty==0 then
					accountItem.Qty[charName]=nil;
					accountItem.ListItem[charName]=nil;
				else
					accountItem.Qty[charName].Subtotal=totQty;
				end
			end
			if inventoryWindow.CharList:GetSelectedIndex()==1 and accountItem.ListItem["ALL"]~=nil then
				if accountItem.Total==0 then
					-- remove the linked display entry
					local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItem.ListItem["ALL"])
					if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
					accountItem.ListItem["ALL"]=nil;
				else
					accountItem.ListItem["ALL"].Qty:SetText(accountItem.Total);
				end
			end
		end

		backpackImage[index].ItemName=nil;
		backpackImage[index].Item=nil;
		backpackImage[index].Index=nil;
		backpackImage[index]=nil;
	end
	CharList[charName].used=CharList[charName].used-1;
	if inventoryWindow.CharList:GetText()==charName then
		inventoryWindow.CapacityDisplay:SetText(Resource[language][5].." ("..tostring(CharList[charName].used).."/"..tostring(CharList[charName].capacity)..")")
	end
end
itemMoved=function(sender,args)
	if Settings.debug then
		Turbine.Shell.WriteLine("itemMoved")
		Table.Dump(args)
		Turbine.Shell.WriteLine("new name:"..backPack:GetItem(args.NewIndex):GetName())
	end
	-- Item, NewIndex, OldIndex
	local itemName=args.Item:GetItemInfo():GetName();
	local newIndex=args.NewIndex
	local newBag=math.floor((newIndex+14)/15);
	local oldIndex=args.OldIndex
	local oldBag=math.floor((oldIndex+14)/15);
	local qty=args.Item:GetQuantity();
	accountItemQty[itemName].Qty[charName][oldIndex]=nil;
	accountItemQty[itemName].Qty[charName][newIndex]=qty;
	if Settings.totalsOnly then
		-- nothing to do
	else
		if accountItemQty[itemName].ListItem[charName]~=nil and accountItemQty[itemName].ListItem[charName][oldIndex]~=nil then
			if accountItemQty[itemName].ListItem[charName][newIndex]~=nil then
				-- we are in the middle of a drag/drop and there has been a bogus "itemAdded" event fired
				-- remove the extraneous bogus element from the ItemList
				local tmpIndex=inventoryPanel.ItemList:IndexOfItem(accountItemQty[itemName].ListItem[charName][newIndex])
				if tmpIndex~=nil and tmpIndex>0 then inventoryPanel.ItemList:RemoveItemAt(tmpIndex) end;
			end
			accountItemQty[itemName].ListItem[charName][newIndex]=accountItemQty[itemName].ListItem[charName][oldIndex];
			accountItemQty[itemName].ListItem[charName][newIndex].Item.Index=newIndex;
			accountItemQty[itemName].ListItem[charName][oldIndex]=nil;
		end
	end
	if backpackImage[oldIndex]~=nil and backpackImage[oldIndex].Item~=nil then
		-- safely remove old callbacks
		if backpackImage[oldIndex].Item.QuantityChanged~=nil then
			RemoveCallback(backpackImage[oldIndex].Item,"QuantityChanged",quantityChanged)
		end
		backpackImage[oldIndex].ItemName=nil;
		backpackImage[oldIndex].Item=nil;
		backpackImage[oldIndex].Index=nil;
		backpackImage[oldIndex]=nil;
	end
	if backpackImage[newIndex]~=nil and backpackImage[newIndex].Item~=nil then
		-- safely remove old callbacks
		if backpackImage[newIndex].Item.QuantityChanged~=nil then
			RemoveCallback(backpackImage[newIndex].Item,"QuantityChanged",quantityChanged)
		end
		backpackImage[newIndex].ItemName=nil;
		backpackImage[newIndex].Item=nil;
		backpackImage[newIndex].Index=nil;
		backpackImage[newIndex]=nil;
	end

	backpackImage[oldIndex]={};
	backpackImage[oldIndex].Item=backPack:GetItem(oldIndex);
	if backpackImage[oldIndex].Item~=nil then
		-- should never happen
		backpackImage[oldIndex].Index=oldIndex;
		AddCallback(backpackImage[oldIndex].Item,"QuantityChanged",quantityChanged);
		backpackImage[oldIndex].Item.Index=oldIndex;
		backpackImage[oldIndex].ItemName=backpackImage[oldIndex].Item:GetItemInfo():GetName();
	end
	backpackImage[newIndex]={}
	backpackImage[newIndex].Item=backPack:GetItem(newIndex);
	if backpackImage[newIndex].Item~=nil then
		backpackImage[newIndex].Index=newIndex;
		AddCallback(backpackImage[newIndex].Item,"QuantityChanged",quantityChanged)
		backpackImage[newIndex].Item.Index=newIndex;
		backpackImage[newIndex].ItemName=backpackImage[newIndex].Item:GetItemInfo():GetName()
	end
end
AddCallback(backPack,"ItemAdded",itemAdded)
AddCallback(backPack,"ItemRemoved",itemRemoved)
AddCallback(backPack,"ItemMoved",itemMoved)

accountItemQty=PatchDataLoad(Turbine.DataScope.Server, "AltInventoryData");
function LoadData()
	if accountItemQty==nil then accountItemQty={} end

	-- remove any data from current character
	for k,v in pairs(accountItemQty) do
		local qty=v.Qty[charName];
		local total=0;
		if qty~=nil then
			accountItemQty[k].Qty[charName]=nil;
		end
		for name,qty in pairs(v.Qty) do
			if name~=charName then
				if type(qty)=="table" then
					total=total+qty.Subtotal;
				else
					total=total+qty;
				end
			end
		end
		accountItemQty[k].Total=total;
		if accountItemQty[k].Total==0 then
			accountItemQty[k]=nil;
		else
			if accountItemQty[k].SortCategory==0 then
				-- attempt to resolve the sort category
				for index,cat in ipairs(ItemCategory) do
					if accountItemQty[k].Category==cat[1] then
						accountItemQty[k].SortCategory=accountItemQty[k].Category;
						break;
					end
				end
			end
		end
	end
	-- now re-add current character bag items
	local tmpIndex;
	for tmpIndex=1,backPack:GetSize() do
		local tmpItem=backPack:GetItem(tmpIndex);
		if tmpItem~=nil then
			local itemInfo=tmpItem:GetItemInfo();
			local name=itemInfo:GetName();
			local qty=tmpItem:GetQuantity();
			if accountItemQty[name]==nil then
				accountItemQty[name]={}
				accountItemQty[name].Total=0;
				accountItemQty[name].BackgroundImageID=itemInfo:GetBackgroundImageID();
				accountItemQty[name].IconImageID=itemInfo:GetIconImageID();
				accountItemQty[name].Quality=GetQualityCode(itemInfo:GetQuality());
				accountItemQty[name].Category=GetCategoryCode(itemInfo:GetCategory());
				accountItemQty[name].SortCategory=GetCategoryCode(itemInfo:GetCategory());
				accountItemQty[name].Qty={};
				accountItemQty[name].ListItem={};
				local found=false;
				for filterIndex=2,inventoryPanel.FilterList.ListData:GetItemCount() do
					if accountItemQty[name].Category==inventoryPanel.FilterList.ListData:GetItem(filterIndex).DataValue then
						inventoryPanel.FilterList:ShowEntry(filterIndex);
						found=true;
						break;
					end
				end
				if not found then
					accountItemQty[name].SortCategory=Turbine.Gameplay.ItemCategory.Undefined;
					inventoryPanel.FilterList:ShowEntry(undefinedCategoryIndex);
				end
			end
			if accountItemQty[name].ItemInfo==nil then
				accountItemQty[name].ItemInfo=itemInfo;
			end
			if accountItemQty[name].Qty[charName]==nil then
				accountItemQty[name].Qty[charName]={["Subtotal"]=qty};
			else
				accountItemQty[name].Qty[charName].Subtotal=accountItemQty[name].Qty[charName].Subtotal+qty;
			end
			accountItemQty[name].Qty[charName][tmpIndex]=qty;
			accountItemQty[name].Total=accountItemQty[name].Total+tmpItem:GetQuantity();
		end
	end

	-- initialize the character list
	local minMenu=minimalWindow.CharMenu:GetItems();
	inventoryWindow.CharList:AddItem(Resource[language][12]);
	minMenu:Add(Turbine.UI.MenuItem(Resource[language][12]));
	inventoryWindow.CharList:AddItem(Resource[language][9]);
	minMenu:Add(Turbine.UI.MenuItem(Resource[language][9]));
	local selIndex=1;
	local tmpList={};
	for k,v in pairs(CharList) do
		if k~="Shared Storage" then
			tmpList[#tmpList+1]=k
		end
	end
	table.sort(tmpList)
	for k,v in ipairs(tmpList) do
		if v==charName then selIndex=k+2 end
		inventoryWindow.CharList:AddItem(v);
		minMenu:Add(Turbine.UI.MenuItem(v));
	end
	if Settings.defaultToAllView==true then
		inventoryWindow.CharList:SetSelectedIndex(1);
	else
		inventoryWindow.CharList:SetSelectedIndex(selIndex);
	end

	inventoryWindow.CharList:SelectedIndexChanged();
	for tmpIndex=1,minMenu:GetCount() do
		minMenu:Get(tmpIndex).Click=function(sender,args)
			inventoryWindow.CharList:SetSelectedIndex(tmpIndex);
			-- have to manually trigger the control to update otherwise the "Current" value doesn't update in time
			inventoryWindow.CharList.ListData:SelectedIndexChanged();
			inventoryWindow.CharList:SelectedIndexChanged();
		end
	end
end
detailMenu=Turbine.UI.ContextMenu()
